"""Structured logging utility for awsui."""

import sys
import json
import logging
from datetime import datetime


class StructuredLogger:
    """JSON structured logger that outputs to STDERR."""

    def __init__(self, level: str = "INFO"):
        self.level = getattr(logging, level.upper())
        self.logger = logging.getLogger("awsui")
        self.logger.setLevel(self.level)

        # Remove any existing handlers
        self.logger.handlers.clear()

        # Add stderr handler with JSON formatter
        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(JSONFormatter())
        self.logger.addHandler(handler)

    def _log(self, level: str, action: str, **kwargs):
        """Internal log method."""
        record = {
            "ts": datetime.utcnow().isoformat() + "Z",
            "level": level,
            "action": action,
        }
        record.update(kwargs)

        # Output to stderr as JSON
        json.dump(record, sys.stderr)
        sys.stderr.write("\n")
        sys.stderr.flush()

    def debug(self, action: str, **kwargs):
        """Log debug message."""
        if self.level <= logging.DEBUG:
            self._log("DEBUG", action, **kwargs)

    def info(self, action: str, **kwargs):
        """Log info message."""
        if self.level <= logging.INFO:
            self._log("INFO", action, **kwargs)

    def warning(self, action: str, **kwargs):
        """Log warning message."""
        if self.level <= logging.WARNING:
            self._log("WARNING", action, **kwargs)

    def error(self, action: str, **kwargs):
        """Log error message."""
        if self.level <= logging.ERROR:
            self._log("ERROR", action, **kwargs)


class JSONFormatter(logging.Formatter):
    """JSON formatter for standard logging."""

    def format(self, record: logging.LogRecord) -> str:
        log_data = {
            "ts": datetime.utcnow().isoformat() + "Z",
            "level": record.levelname,
            "action": record.getMessage(),
        }

        # Add extra fields if present
        if hasattr(record, "duration_ms"):
            log_data["duration_ms"] = record.duration_ms
        if hasattr(record, "profile"):
            log_data["profile"] = record.profile
        if hasattr(record, "result"):
            log_data["result"] = record.result

        return json.dumps(log_data)


# Global logger instance
_logger: StructuredLogger | None = None


def get_logger(level: str = "INFO") -> StructuredLogger:
    """Get or create the global logger instance."""
    global _logger
    if _logger is None:
        _logger = StructuredLogger(level)
    return _logger


"""Tests for awsui.service_model_loader."""

import json

from awsui.service_model_loader import ServiceModelLoader


def test_camel_to_kebab_conversion():
    assert ServiceModelLoader._camel_to_kebab("DescribeInstances") == "describe-instances"
    assert ServiceModelLoader._camel_to_kebab("DBCluster") == "db-cluster"


def test_get_service_operations_reads_model(tmp_path):
    loader = ServiceModelLoader()
    service_dir = tmp_path / "ec2" / "2024-01-01"
    service_dir.mkdir(parents=True)
    service_file = service_dir / "service-2.json"
    service_file.write_text(
        json.dumps({
            "operations": {
                "DescribeInstances": {},
                "StartInstances": {},
            }
        }),
        encoding="utf-8",
    )

    loader.botocore_data_path = tmp_path
    loader._service_cache.clear()

    operations = loader.get_service_operations("ec2")
    assert operations == ["describe-instances", "start-instances"]

    # Removing the file should not break cached result on subsequent calls
    service_file.unlink()
    assert loader.get_service_operations("ec2") == operations


def test_get_service_operations_uses_cache(monkeypatch):
    loader = ServiceModelLoader()
    calls = {"count": 0}

    def fake_load(service):
        calls["count"] += 1
        return {"operations": {"ListThings": {}}}

    monkeypatch.setattr(loader, "_load_service_model", fake_load)

    assert loader.get_service_operations("iot") == ["list-things"]
    assert loader.get_service_operations("iot") == ["list-things"]
    assert calls["count"] == 1


def test_get_operation_parameters_returns_cli_names(monkeypatch):
    loader = ServiceModelLoader()

    def fake_load(service):
        return {
            "operations": {
                "DescribeInstances": {
                    "input": {"shape": "DescribeInstancesRequest"}
                }
            },
            "shapes": {
                "DescribeInstancesRequest": {
                    "members": {
                        "InstanceIds": {},
                        "MaxResults": {},
                    }
                }
            },
        }

    monkeypatch.setattr(loader, "_load_service_model", fake_load)

    params = loader.get_operation_parameters("ec2", "describe-instances")
    assert set(params) == {"--instance-ids", "--max-results"}


def test_get_operation_parameters_missing_input(monkeypatch):
    loader = ServiceModelLoader()

    def fake_load(service):
        return {"operations": {"Describe": {}}}

    monkeypatch.setattr(loader, "_load_service_model", fake_load)
    assert loader.get_operation_parameters("ec2", "describe") == []



"""AWS CLI command parser for intelligent autocomplete."""

import re
from dataclasses import dataclass
from enum import Enum

from .service_model_loader import get_service_loader


class CompletionContext(Enum):
    """Context for autocomplete suggestions."""
    SERVICE = "service"           # Completing AWS service name (e.g., "aws s3")
    COMMAND = "command"           # Completing command (e.g., "aws s3 ls")
    PARAMETER = "parameter"       # Completing parameter name (e.g., "--region")
    PARAMETER_VALUE = "value"     # Completing parameter value (e.g., "--region us-")


@dataclass
class ParsedCommand:
    """Parsed AWS CLI command structure."""
    raw_input: str
    service: str = ""
    command: str = ""
    subcommand: str = ""
    parameters: dict[str, str | None] = None
    current_context: CompletionContext = CompletionContext.SERVICE
    current_token: str = ""
    cursor_position: int = 0

    def __post_init__(self):
        if self.parameters is None:
            self.parameters = {}


class AWSCommandParser:
    """Parser for AWS CLI commands to enable context-aware autocomplete.

    Uses a hybrid approach:
    - Fast path: Hardcoded common services and commands
    - Fallback: Dynamic loading from botocore for all AWS services
    """

    SPECIAL_COMMANDS = ["help", "configure"]

    COMMON_SERVICES = [
        "s3", "ec2", "lambda", "dynamodb", "rds", "iam", "sts", "cloudformation",
        "cloudfront", "cloudtrail", "cloudwatch", "logs", "ecr", "ecs", "eks",
        "sns", "sqs", "secretsmanager", "ssm", "stepfunctions", "kinesis", "kms",
        "glue", "organizations", "route53", "redshift"
    ]

    SERVICE_COMMANDS = {
        "s3": ["ls", "cp", "mv", "rm", "sync", "mb", "rb"],
        "ec2": ["describe-instances", "start-instances", "stop-instances", "reboot-instances",
                "describe-security-groups", "describe-vpcs", "describe-subnets", "create-image"],
        "lambda": ["list-functions", "invoke", "get-function", "update-function-code",
                   "create-function", "delete-function"],
        "iam": ["list-users", "list-roles", "get-user", "list-policies", "attach-role-policy"],
        "sts": ["get-caller-identity", "assume-role", "get-session-token"],
        "cloudformation": ["list-stacks", "describe-stacks", "create-stack", "update-stack", "delete-stack"],
        "dynamodb": ["list-tables", "describe-table", "scan", "query", "update-table"],
        "ecr": ["describe-repositories", "get-login-password", "list-images", "batch-delete-image"],
        "ecs": ["list-clusters", "list-services", "describe-services", "update-service"],
        "eks": ["list-clusters", "describe-cluster", "update-kubeconfig"],
        "rds": ["describe-db-instances", "describe-db-clusters", "create-db-snapshot"],
        "logs": ["tail", "describe-log-groups", "describe-log-streams"],
        "cloudwatch": ["list-metrics", "get-metric-statistics", "describe-alarms"],
    }

    GLOBAL_PARAMETERS = [
        "--version",
        "--debug",
        "--no-verify-ssl",
        "--no-paginate",
        "--output",
        "--query",
        "--profile",
        "--region",
        "--endpoint-url",
        "--no-cli-pager",
        "--color",
        "--no-sign-request",
        "--ca-bundle",
        "--cli-read-timeout",
        "--cli-connect-timeout",
    ]

    COMMON_PARAMETERS = [
        "--region",
        "--output",
        "--profile",
        "--query",
        "--no-cli-pager",
        "--no-verify-ssl",
        "--endpoint-url",
        "--debug",
        "--no-paginate",
        "--max-items",
        "--page-size",
    ]

    PARAMETER_VALUES = {
        "--region": [
            "us-east-1", "us-east-2", "us-west-1", "us-west-2",
            "eu-west-1", "eu-west-2", "eu-west-3", "eu-central-1",
            "ap-southeast-1", "ap-southeast-2", "ap-northeast-1", "ap-northeast-2",
            "sa-east-1", "ca-central-1"
        ],
        "--output": ["json", "yaml", "yaml-stream", "text", "table"],
    }

    SERVICE_PARAMETERS = {
        "s3": ["--recursive", "--exclude", "--include", "--delete", "--acl", "--storage-class"],
        "ec2": ["--instance-ids", "--security-group-ids", "--vpc-id", "--subnet-id", "--filters"],
        "lambda": ["--function-name", "--payload", "--zip-file", "--runtime", "--handler", "--role"],
        "iam": ["--role-name", "--policy-arn", "--user-name", "--group-name"],
        "cloudformation": ["--stack-name", "--template-body", "--template-url", "--parameters"],
        "dynamodb": ["--table-name", "--key", "--attribute-definitions", "--provisioned-throughput"],
    }

    def __init__(self):
        """Initialize parser with optional dynamic service loader."""
        self.service_loader = get_service_loader()
        self.use_dynamic_loading = self.service_loader.is_available()

    def parse(self, command_line: str, cursor_pos: int | None = None) -> ParsedCommand:
        """
        Parse AWS CLI command and determine current context for autocomplete.

        Args:
            command_line: The command line input
            cursor_pos: Cursor position (defaults to end of string)

        Returns:
            ParsedCommand with parsed structure and context
        """
        if cursor_pos is None:
            cursor_pos = len(command_line)

        text_to_cursor = command_line[:cursor_pos]
        tokens = self._tokenize(text_to_cursor)

        parsed = ParsedCommand(
            raw_input=command_line,
            cursor_position=cursor_pos
        )

        if not tokens:
            parsed.current_context = CompletionContext.SERVICE
            return parsed

        if tokens[0].lower() == "aws":
            tokens = tokens[1:]

        if not tokens:
            parsed.current_context = CompletionContext.SERVICE
            return parsed

        is_completing_new = text_to_cursor.endswith(" ")
        current_token = "" if is_completing_new else tokens[-1] if tokens else ""
        parsed.current_token = current_token

        if len(tokens) >= 1:
            potential_service = tokens[0]
            if self._is_valid_service(potential_service):
                parsed.service = potential_service
            elif not potential_service.startswith("-"):
                parsed.current_context = CompletionContext.SERVICE
                return parsed

        if len(tokens) >= 2 and parsed.service:
            potential_command = tokens[1]
            if not potential_command.startswith("-"):
                parsed.command = potential_command
                parsed.current_context = CompletionContext.COMMAND
            else:
                parsed.current_context = CompletionContext.COMMAND
                return parsed

        i = 2 if parsed.command else (1 if parsed.service else 0)
        expecting_value = None

        while i < len(tokens):
            token = tokens[i]

            if token.startswith("--"):
                if "=" in token:
                    param, value = token.split("=", 1)
                    parsed.parameters[param] = value
                else:
                    parsed.parameters[token] = None
                    expecting_value = token
            elif token.startswith("-") and len(token) == 2:
                parsed.parameters[token] = None
                expecting_value = token
            else:
                if expecting_value:
                    parsed.parameters[expecting_value] = token
                    expecting_value = None

            i += 1

        if is_completing_new or (tokens and not tokens[-1].startswith("-")):
            if expecting_value:
                parsed.current_context = CompletionContext.PARAMETER_VALUE
                parsed.current_token = current_token if not is_completing_new else ""
            elif parsed.service and parsed.command and is_completing_new:
                parsed.current_context = CompletionContext.PARAMETER
                parsed.current_token = ""
            elif parsed.service and not is_completing_new:
                parsed.current_context = CompletionContext.COMMAND
                parsed.current_token = current_token
            elif parsed.service:
                parsed.current_context = CompletionContext.COMMAND
                parsed.current_token = ""
            else:
                parsed.current_context = CompletionContext.SERVICE
                parsed.current_token = current_token if not is_completing_new else ""
        else:
            if current_token.startswith("--") or current_token.startswith("-"):
                parsed.current_context = CompletionContext.PARAMETER
            elif not parsed.service:
                parsed.current_context = CompletionContext.SERVICE
            elif not parsed.command:
                parsed.current_context = CompletionContext.COMMAND
            else:
                parsed.current_context = CompletionContext.PARAMETER

        return parsed

    def _tokenize(self, text: str) -> list[str]:
        """Tokenize command line, respecting quotes."""
        tokens = []
        current_token = ""
        in_quote = None

        for char in text:
            if char in ('"', "'"):
                if in_quote == char:
                    in_quote = None
                elif in_quote is None:
                    in_quote = char
                current_token += char
            elif char.isspace() and in_quote is None:
                if current_token:
                    tokens.append(current_token)
                    current_token = ""
            else:
                current_token += char

        if current_token:
            tokens.append(current_token)

        return tokens

    def _is_valid_service(self, service: str) -> bool:
        """Check if service is valid (hardcoded or from botocore)."""
        # Special commands (help, configure) are treated as "services" for parsing
        if service in self.SPECIAL_COMMANDS:
            return True
        if service in self.COMMON_SERVICES:
            return True
        if self.use_dynamic_loading:
            return service in self.service_loader.get_all_services()
        return False

    def _get_all_services(self) -> list[str]:
        """Get all available services (hybrid: hardcoded + dynamic)."""
        services = set(self.COMMON_SERVICES)
        if self.use_dynamic_loading:
            services.update(self.service_loader.get_all_services())
        return sorted(services)

    def _get_service_commands(self, service: str) -> list[str]:
        """Get commands for a service (hybrid: dynamic preferred, hardcoded fallback)."""
        if self.use_dynamic_loading:
            commands = self.service_loader.get_service_operations(service)
            if commands:
                return commands

        if service in self.SERVICE_COMMANDS:
            return self.SERVICE_COMMANDS[service]

        return []

    def _get_command_parameters(self, service: str, command: str) -> list[str]:
        """Get parameters for a command (hybrid: common + service-specific + dynamic)."""
        suggestions = list(self.COMMON_PARAMETERS)

        if service in self.SERVICE_PARAMETERS:
            suggestions.extend(self.SERVICE_PARAMETERS[service])

        if self.use_dynamic_loading:
            dynamic_params = self.service_loader.get_operation_parameters(service, command)
            suggestions.extend(dynamic_params)

        return list(set(suggestions))

    def get_suggestions(self, parsed: ParsedCommand) -> list[str]:
        """
        Get autocomplete suggestions based on parsed command context.

        Uses hybrid approach: hardcoded (fast) + dynamic (complete).

        Args:
            parsed: Parsed command structure

        Returns:
            List of suggestion strings
        """
        query = parsed.current_token.lower()

        if parsed.current_context == CompletionContext.SERVICE:
            all_services = self._get_all_services()
            services = [s for s in all_services if s.startswith(query)]
            special_cmds = [c for c in self.SPECIAL_COMMANDS if c.startswith(query)]

            if query.startswith("-"):
                global_params = [p for p in self.GLOBAL_PARAMETERS if p.startswith(query)]
                return global_params + special_cmds + services

            return special_cmds + services

        elif parsed.current_context == CompletionContext.COMMAND:
            if parsed.service in self.SPECIAL_COMMANDS:
                return []
            commands = self._get_service_commands(parsed.service)
            return [c for c in commands if c.startswith(query)]

        elif parsed.current_context == CompletionContext.PARAMETER:
            if parsed.service in self.SPECIAL_COMMANDS:
                return []

            if not parsed.service:
                return [p for p in self.GLOBAL_PARAMETERS if p.startswith(query)]

            suggestions = self._get_command_parameters(parsed.service, parsed.command)
            used_params = set(parsed.parameters.keys())
            suggestions = [p for p in suggestions if p not in used_params]

            return [p for p in suggestions if p.startswith(query)]

        elif parsed.current_context == CompletionContext.PARAMETER_VALUE:
            last_param = None
            for param, value in reversed(list(parsed.parameters.items())):
                if value is None:
                    last_param = param
                    break

            if last_param and last_param in self.PARAMETER_VALUES:
                values = self.PARAMETER_VALUES[last_param]
                return [v for v in values if v.startswith(query)]

        return []



"""Pytest configuration and fixtures."""

import pytest
import os


@pytest.fixture
def temp_aws_config(tmp_path):
    """Create a temporary AWS config file."""
    config_dir = tmp_path / ".aws"
    config_dir.mkdir()
    config_file = config_dir / "config"
    return config_file


@pytest.fixture
def temp_aws_credentials(tmp_path):
    """Create a temporary AWS credentials file."""
    config_dir = tmp_path / ".aws"
    config_dir.mkdir(exist_ok=True)
    credentials_file = config_dir / "credentials"
    return credentials_file


@pytest.fixture
def sample_sso_config():
    """Return sample SSO configuration content."""
    return """[sso-session corp]
sso_start_url = https://example.awsapps.com/start
sso_region = ap-northeast-1

[profile test-sso]
sso_session = corp
sso_account_id = 111111111111
sso_role_name = AdministratorAccess
region = ap-northeast-1
output = json
"""


@pytest.fixture
def sample_assume_config():
    """Return sample assume role configuration content."""
    return """[profile base]
region = us-east-1

[profile test-assume]
source_profile = base
role_arn = arn:aws:iam::222222222222:role/MyRole
region = us-west-2
"""


@pytest.fixture
def mock_env(monkeypatch):
    """Provide a clean environment for testing."""
    # Clear AWS-related environment variables
    for key in list(os.environ.keys()):
        if key.startswith("AWS_"):
            monkeypatch.delenv(key, raising=False)

    return monkeypatch


"""Configuration file parsing and discovery."""

import os
from pathlib import Path
from configparser import ConfigParser
from typing import List
from .models import Profile


def get_config_paths() -> tuple[Path, Path]:
    """
    Get AWS config and credentials file paths.

    Honors AWS_CONFIG_FILE and AWS_SHARED_CREDENTIALS_FILE environment variables.
    """
    home = Path.home()

    config_path = os.environ.get("AWS_CONFIG_FILE")
    if config_path:
        config_file = Path(config_path)
    else:
        config_file = home / ".aws" / "config"

    credentials_path = os.environ.get("AWS_SHARED_CREDENTIALS_FILE")
    if credentials_path:
        credentials_file = Path(credentials_path)
    else:
        credentials_file = home / ".aws" / "credentials"

    return config_file, credentials_file


def parse_profiles() -> List[Profile]:
    """
    Parse AWS profiles from config and credentials files.

    Returns a list of Profile objects.
    """
    config_file, credentials_file = get_config_paths()
    profiles: List[Profile] = []

    # Parse config file
    if config_file.exists():
        config = ConfigParser()
        config.read(config_file)

        for section in config.sections():
            # Skip sso-session sections
            if section.startswith("sso-session "):
                continue

            # Profile sections start with "profile "
            if section.startswith("profile "):
                profile_name = section[8:]  # Remove "profile " prefix
            else:
                # Default profile
                profile_name = section

            section_data = dict(config[section])

            # Determine profile kind
            if "sso_session" in section_data or "sso_start_url" in section_data:
                kind = "sso"
                session = section_data.get("sso_session")
                account = section_data.get("sso_account_id")
                role = section_data.get("sso_role_name")
            elif "source_profile" in section_data and "role_arn" in section_data:
                kind = "assume"
                session = None
                # Extract account from role_arn if present
                role_arn = section_data.get("role_arn", "")
                account = role_arn.split(":")[4] if ":" in role_arn else None
                role = role_arn.split("/")[-1] if "/" in role_arn else None
            else:
                kind = "basic"
                session = None
                account = None
                role = None

            profile = Profile(
                name=profile_name,
                kind=kind,
                account=account,
                role=role,
                region=section_data.get("region"),
                session=session,
                source=str(config_file)
            )
            profiles.append(profile)

    # Parse credentials file for basic profiles
    if credentials_file.exists():
        creds = ConfigParser()
        creds.read(credentials_file)

        for section in creds.sections():
            # Check if this profile already exists from config
            if any(p["name"] == section for p in profiles):
                continue

            # This is a basic credential profile
            profile = Profile(
                name=section,
                kind="basic",
                account=None,
                role=None,
                region=None,
                session=None,
                source=str(credentials_file)
            )
            profiles.append(profile)

    return profiles


"""Tests for special commands not showing autocomplete suggestions."""

import pytest
from awsui.command_parser import AWSCommandParser, CompletionContext


@pytest.fixture
def parser():
    parser = AWSCommandParser()
    parser.use_dynamic_loading = False
    return parser


def test_help_with_trailing_space_no_suggestions(parser):
    """Test 'aws help ' does not show command suggestions."""
    parsed = parser.parse("aws help ")
    assert parsed.service == "help"
    assert parsed.current_context == CompletionContext.COMMAND
    suggestions = parser.get_suggestions(parsed)
    assert suggestions == []


def test_help_with_parameter_no_suggestions(parser):
    """Test 'aws help --' does not show parameter suggestions."""
    parsed = parser.parse("aws help --")
    assert parsed.service == "help"
    suggestions = parser.get_suggestions(parsed)
    assert suggestions == []


def test_help_typing_text_no_suggestions(parser):
    """Test 'aws help something' does not show suggestions."""
    parsed = parser.parse("aws help something")
    assert parsed.service == "help"
    suggestions = parser.get_suggestions(parsed)
    assert suggestions == []


def test_configure_with_trailing_space_no_suggestions(parser):
    """Test 'aws configure ' does not show command suggestions."""
    parsed = parser.parse("aws configure ")
    assert parsed.service == "configure"
    assert parsed.current_context == CompletionContext.COMMAND
    suggestions = parser.get_suggestions(parsed)
    assert suggestions == []


def test_configure_with_parameter_no_suggestions(parser):
    """Test 'aws configure --' does not show parameter suggestions."""
    parsed = parser.parse("aws configure --")
    assert parsed.service == "configure"
    suggestions = parser.get_suggestions(parsed)
    assert suggestions == []


def test_help_recognized_as_service(parser):
    """Test 'help' is recognized as valid service."""
    parsed = parser.parse("aws help")
    assert parsed.service == "help"
    assert parser._is_valid_service("help") is True


def test_configure_recognized_as_service(parser):
    """Test 'configure' is recognized as valid service."""
    parsed = parser.parse("aws configure")
    assert parsed.service == "configure"
    assert parser._is_valid_service("configure") is True


def test_special_commands_in_constant(parser):
    """Test special commands are defined in SPECIAL_COMMANDS."""
    assert "help" in parser.SPECIAL_COMMANDS
    assert "configure" in parser.SPECIAL_COMMANDS



"""Tests for awsui.resource_suggester."""

from datetime import datetime, timedelta

from awsui.resource_suggester import ResourceCache, ResourceSuggester


def test_resource_cache_respects_ttl(monkeypatch):
    cache = ResourceCache(ttl_seconds=10)
    start_time = datetime(2024, 1, 1, 0, 0, 0)

    class FrozenDatetime(datetime):
        @classmethod
        def now(cls):
            return start_time

    monkeypatch.setattr("awsui.resource_suggester.datetime", FrozenDatetime)
    cache.set("key", ["value"])
    assert cache.get("key") == ["value"]

    class SlightlyLaterDatetime(datetime):
        @classmethod
        def now(cls):
            return start_time + timedelta(seconds=5)

    monkeypatch.setattr("awsui.resource_suggester.datetime", SlightlyLaterDatetime)
    assert cache.get("key") == ["value"]

    class ExpiredDatetime(datetime):
        @classmethod
        def now(cls):
            return start_time + timedelta(seconds=11)

    monkeypatch.setattr("awsui.resource_suggester.datetime", ExpiredDatetime)
    assert cache.get("key") is None


def test_run_aws_command_appends_context(monkeypatch):
    captured = {}

    def fake_run(args, capture_output, text, timeout, check):
        captured["args"] = list(args)

        class Result:
            returncode = 0
            stdout = "[]"

        return Result()

    monkeypatch.setattr("awsui.resource_suggester.subprocess.run", fake_run)
    suggester = ResourceSuggester(profile="dev", region="us-west-2")
    command = ["aws", "ec2", "describe-instances"]

    output = suggester._run_aws_command(command)

    assert output == "[]"
    assert command == [
        "aws",
        "ec2",
        "describe-instances",
        "--profile",
        "dev",
        "--region",
        "us-west-2",
        "--output",
        "json",
    ]
    assert captured["args"] == command


def test_get_suggestions_for_parameter_uses_mapping(monkeypatch):
    suggester = ResourceSuggester()
    calls = {}

    def fake_instances():
        calls["instances"] = True
        return ["i-123"]

    monkeypatch.setattr(suggester, "get_ec2_instance_ids", fake_instances)
    suggestions = suggester.get_suggestions_for_parameter(
        "ec2", "describe-instances", "--instance-ids"
    )

    assert calls == {"instances": True}
    assert suggestions == ["i-123"]


def test_get_suggestions_for_parameter_s3(monkeypatch):
    suggester = ResourceSuggester()
    monkeypatch.setattr(suggester, "get_s3_buckets", lambda: ["bucket-a"])

    assert suggester.get_suggestions_for_parameter("s3", "ls", "--any") == ["bucket-a"]



"""Tests for config parsing."""

import os
from pathlib import Path
from tempfile import NamedTemporaryFile
from awsui.config import parse_profiles


def test_parse_sso_profile():
    """Test parsing SSO profile from config."""
    config_content = """[sso-session corp]
sso_start_url = https://example.awsapps.com/start
sso_region = ap-northeast-1

[profile test-sso]
sso_session = corp
sso_account_id = 111111111111
sso_role_name = AdministratorAccess
region = ap-northeast-1
output = json
"""

    with NamedTemporaryFile(mode='w', suffix='.config', delete=False) as f:
        f.write(config_content)
        config_path = f.name

    try:
        # Set environment variable to use test config
        original_config = os.environ.get("AWS_CONFIG_FILE")
        os.environ["AWS_CONFIG_FILE"] = config_path

        profiles = parse_profiles()

        assert len(profiles) >= 1
        sso_profile = next((p for p in profiles if p["name"] == "test-sso"), None)
        assert sso_profile is not None
        assert sso_profile["kind"] == "sso"
        assert sso_profile["account"] == "111111111111"
        assert sso_profile["role"] == "AdministratorAccess"
        assert sso_profile["region"] == "ap-northeast-1"
        assert sso_profile["session"] == "corp"

    finally:
        # Restore environment
        if original_config:
            os.environ["AWS_CONFIG_FILE"] = original_config
        elif "AWS_CONFIG_FILE" in os.environ:
            del os.environ["AWS_CONFIG_FILE"]

        # Clean up temp file
        Path(config_path).unlink(missing_ok=True)


def test_parse_assume_role_profile():
    """Test parsing assume role profile."""
    config_content = """[profile base]
region = us-east-1

[profile test-assume]
source_profile = base
role_arn = arn:aws:iam::222222222222:role/MyRole
region = us-west-2
"""

    with NamedTemporaryFile(mode='w', suffix='.config', delete=False) as f:
        f.write(config_content)
        config_path = f.name

    try:
        original_config = os.environ.get("AWS_CONFIG_FILE")
        os.environ["AWS_CONFIG_FILE"] = config_path

        profiles = parse_profiles()

        assume_profile = next((p for p in profiles if p["name"] == "test-assume"), None)
        assert assume_profile is not None
        assert assume_profile["kind"] == "assume"
        assert assume_profile["account"] == "222222222222"
        assert assume_profile["role"] == "MyRole"
        assert assume_profile["region"] == "us-west-2"

    finally:
        if original_config:
            os.environ["AWS_CONFIG_FILE"] = original_config
        elif "AWS_CONFIG_FILE" in os.environ:
            del os.environ["AWS_CONFIG_FILE"]

        Path(config_path).unlink(missing_ok=True)


def test_parse_empty_config():
    """Test parsing empty config file."""
    with NamedTemporaryFile(mode='w', suffix='.config', delete=False) as f:
        f.write("")
        config_path = f.name

    try:
        original_config = os.environ.get("AWS_CONFIG_FILE")
        os.environ["AWS_CONFIG_FILE"] = config_path

        profiles = parse_profiles()
        # Should return empty list or only profiles from credentials file
        assert isinstance(profiles, list)

    finally:
        if original_config:
            os.environ["AWS_CONFIG_FILE"] = original_config
        elif "AWS_CONFIG_FILE" in os.environ:
            del os.environ["AWS_CONFIG_FILE"]

        Path(config_path).unlink(missing_ok=True)


"""Tests for AWS CLI command parser."""

import pytest
from awsui.command_parser import AWSCommandParser, CompletionContext


class TestAWSCommandParser:
    """Test AWS CLI command parsing functionality."""

    @pytest.fixture
    def parser(self):
        parser = AWSCommandParser()
        parser.use_dynamic_loading = False
        return parser

    def test_parse_service_completion(self, parser):
        """Test service name completion."""
        parsed = parser.parse("aws s")
        assert parsed.current_context == CompletionContext.SERVICE
        assert parsed.current_token == "s"
        assert parsed.service == ""

    def test_parse_service_selected(self, parser):
        """Test after service is selected."""
        parsed = parser.parse("aws s3 ")
        assert parsed.current_context == CompletionContext.COMMAND
        assert parsed.service == "s3"
        assert parsed.command == ""

    def test_parse_command_completion(self, parser):
        """Test command completion."""
        parsed = parser.parse("aws s3 l")
        assert parsed.current_context == CompletionContext.COMMAND
        assert parsed.service == "s3"
        assert parsed.current_token == "l"

    def test_parse_parameter_suggestion(self, parser):
        """Test parameter suggestion after command."""
        parsed = parser.parse("aws s3 ls ")
        assert parsed.current_context == CompletionContext.PARAMETER
        assert parsed.service == "s3"
        assert parsed.command == "ls"

    def test_parse_parameter_completion(self, parser):
        """Test parameter name completion."""
        parsed = parser.parse("aws ec2 describe-instances --reg")
        assert parsed.current_context == CompletionContext.PARAMETER
        assert parsed.current_token == "--reg"
        assert parsed.command == "describe-instances"

    def test_parse_parameter_value_completion(self, parser):
        """Parameter value context is active while awaiting a value."""
        parsed = parser.parse("aws ec2 describe-instances --region ")
        assert parsed.current_context == CompletionContext.PARAMETER_VALUE
        assert parsed.current_token == ""
        assert parsed.parameters["--region"] is None

    def test_get_service_suggestions(self, parser):
        """Test service name suggestions."""
        parsed = parser.parse("aws s")
        suggestions = parser.get_suggestions(parsed)
        assert "s3" in suggestions
        assert "sns" in suggestions
        assert "sqs" in suggestions

    def test_get_command_suggestions(self, parser):
        """Test command suggestions for service."""
        parsed = parser.parse("aws s3 l")
        suggestions = parser.get_suggestions(parsed)
        assert "ls" in suggestions

    def test_get_parameter_suggestions(self, parser):
        """Test parameter suggestions."""
        parsed = parser.parse("aws s3 ls --")
        suggestions = parser.get_suggestions(parsed)
        assert "--region" in suggestions
        assert "--output" in suggestions
        assert "--recursive" in suggestions

    def test_get_region_value_suggestions(self, parser):
        """Test region value suggestions."""
        parsed = parser.parse("aws ec2 describe-instances --region ")
        suggestions = parser.get_suggestions(parsed)
        assert "us-east-1" in suggestions
        assert "us-west-2" in suggestions

    def test_multiple_parameters(self, parser):
        """Test parsing multiple parameters."""
        parsed = parser.parse("aws ec2 describe-instances --region us-east-1 --output json ")
        assert parsed.current_context == CompletionContext.PARAMETER
        assert parsed.parameters["--region"] == "us-east-1"
        assert parsed.parameters["--output"] == "json"

    def test_tokenization_with_quotes(self, parser):
        """Test tokenization handles quotes."""
        parsed = parser.parse('aws s3 cp "file with spaces.txt" s3://bucket/')
        assert parsed.service == "s3"
        assert parsed.command == "cp"



"""AWS CLI wrapper for STS and SSO operations."""

import json
import subprocess
import time
from typing import Any, Callable, Dict


def check_aws_cli_available() -> bool:
    """Check if AWS CLI v2 is available."""
    try:
        result = subprocess.run(
            ["aws", "--version"],
            capture_output=True,
            text=True,
            timeout=5
        )
        return result.returncode == 0 and "aws-cli/2" in result.stdout
    except (subprocess.SubprocessError, FileNotFoundError):
        return False


def get_caller_identity(profile: str) -> Dict[str, Any] | None:
    """
    Execute 'aws sts get-caller-identity' for the given profile.

    Returns parsed JSON response or None on failure.
    """
    try:
        result = subprocess.run(
            ["aws", "sts", "get-caller-identity", "--profile", profile],
            capture_output=True,
            text=True,
            timeout=10
        )
        if result.returncode == 0:
            return json.loads(result.stdout)
        return None
    except (subprocess.SubprocessError, json.JSONDecodeError):
        return None


def _terminate_process(process: subprocess.Popen[Any]) -> None:
    """Terminate process gracefully, falling back to kill if needed."""
    try:
        process.terminate()
    except Exception:
        return

    try:
        process.wait(timeout=5)
    except subprocess.TimeoutExpired:
        try:
            process.kill()
        except Exception:
            return
        try:
            process.wait(timeout=5)
        except subprocess.TimeoutExpired:
            pass


def sso_login(
    profile: str,
    cancel_check: Callable[[], bool] | None = None,
    timeout: int = 300,
    poll_interval: float = 0.5
) -> bool:
    """
    Execute 'aws sso login --profile <name>' with optional cancellation.

    Returns True if login succeeded and False otherwise.
    """
    try:
        process = subprocess.Popen(
            ["aws", "sso", "login", "--profile", profile],
            stdout=subprocess.DEVNULL,
            stderr=subprocess.STDOUT
        )
    except (OSError, ValueError, subprocess.SubprocessError):
        return False

    start_time = time.monotonic()

    while True:
        if cancel_check and cancel_check():
            _terminate_process(process)
            return False

        result = process.poll()
        if result is not None:
            return result == 0

        if timeout and (time.monotonic() - start_time) >= timeout:
            _terminate_process(process)
            return False

        time.sleep(poll_interval)


def ensure_authenticated(
    profile: str,
    cancel_check: Callable[[], bool] | None = None
) -> Dict[str, Any] | None:
    """
    Ensure profile is authenticated, performing SSO login if needed.

    Returns caller identity on success, None on failure.
    """
    if cancel_check and cancel_check():
        return None

    identity = get_caller_identity(profile)
    if identity:
        return identity

    if cancel_check and cancel_check():
        return None

    if sso_login(profile, cancel_check=cancel_check):
        if cancel_check and cancel_check():
            return None
        return get_caller_identity(profile)

    return None



"""Dynamic AWS service model loader from botocore."""

import json
import re
import sys
from pathlib import Path
from typing import Dict, List, Optional
from functools import lru_cache


class ServiceModelLoader:
    """Load AWS service models dynamically from botocore data files."""

    def __init__(self):
        self.botocore_data_path: Optional[Path] = None
        self._service_cache: Dict[str, dict] = {}
        self._find_botocore_path()

    def _find_botocore_path(self) -> None:
        """Find botocore data directory by checking common locations."""
        possible_paths = []

        # Try to import botocore and get its path
        try:
            import botocore
            botocore_module_path = Path(botocore.__file__).parent
            possible_paths.append(botocore_module_path / "data")
        except ImportError:
            pass

        # Check AWS CLI v2 bundled botocore (Homebrew on macOS)
        if sys.platform == "darwin":
            homebrew_paths = [
                Path("/opt/homebrew/Cellar/awscli"),
                Path("/usr/local/Cellar/awscli"),
            ]
            for base_path in homebrew_paths:
                if base_path.exists():
                    for version_dir in base_path.glob("*/libexec/lib/python*/site-packages/awscli/botocore/data"):
                        possible_paths.append(version_dir)

        # Check common Linux installation paths
        if sys.platform.startswith("linux"):
            linux_paths = [
                Path("/usr/local/aws-cli/v2/current/dist/awscli/botocore/data"),
                Path("/usr/lib/python*/site-packages/botocore/data"),
            ]
            for pattern in linux_paths:
                for path in Path("/").glob(str(pattern).lstrip("/")):
                    possible_paths.append(path)

        # Check user's site-packages
        try:
            import site
            for site_dir in site.getsitepackages():
                possible_paths.append(Path(site_dir) / "botocore" / "data")
                possible_paths.append(Path(site_dir) / "awscli" / "botocore" / "data")
        except Exception:
            pass

        # Find the first valid path
        for path in possible_paths:
            if path.exists() and (path / "ec2").exists():
                self.botocore_data_path = path
                return

        # Fallback: assume botocore is not available
        self.botocore_data_path = None

    def is_available(self) -> bool:
        """Check if botocore data is available."""
        return self.botocore_data_path is not None

    @lru_cache(maxsize=512)
    def get_all_services(self) -> List[str]:
        """Get list of all available AWS services."""
        if not self.botocore_data_path:
            return []

        services = []
        for service_dir in self.botocore_data_path.iterdir():
            if service_dir.is_dir() and not service_dir.name.startswith("."):
                services.append(service_dir.name)

        return sorted(services)

    def _load_service_model(self, service: str) -> Optional[dict]:
        """Load service-2.json for a given service."""
        if not self.botocore_data_path:
            return None

        if service in self._service_cache:
            return self._service_cache[service]

        service_path = self.botocore_data_path / service
        if not service_path.exists():
            return None

        # Find the latest version directory
        version_dirs = [d for d in service_path.iterdir() if d.is_dir()]
        if not version_dirs:
            return None

        # Use the first (usually only) version
        version_dir = version_dirs[0]
        service_file = version_dir / "service-2.json"

        if not service_file.exists():
            return None

        try:
            with open(service_file, "r", encoding="utf-8") as f:
                model = json.load(f)
                self._service_cache[service] = model
                return model
        except (json.JSONDecodeError, IOError):
            return None

    @staticmethod
    def _camel_to_kebab(name: str) -> str:
        """Convert CamelCase to kebab-case."""
        # Insert hyphen before uppercase letters (except at start)
        s1 = re.sub("(.)([A-Z][a-z]+)", r"\1-\2", name)
        # Insert hyphen before uppercase letters followed by lowercase
        s2 = re.sub("([a-z0-9])([A-Z])", r"\1-\2", s1)
        return s2.lower()

    @lru_cache(maxsize=512)
    def get_service_operations(self, service: str) -> List[str]:
        """Get all operations (commands) for a service in kebab-case."""
        model = self._load_service_model(service)
        if not model or "operations" not in model:
            return []

        operations = model["operations"].keys()
        # Convert CamelCase to kebab-case for CLI
        return sorted([self._camel_to_kebab(op) for op in operations])

    @lru_cache(maxsize=1024)
    def get_operation_parameters(self, service: str, operation: str) -> List[str]:
        """Get parameters for a specific operation."""
        model = self._load_service_model(service)
        if not model or "operations" not in model:
            return []

        # Convert kebab-case back to CamelCase to find in model
        operation_camel = "".join(word.capitalize() for word in operation.split("-"))

        if operation_camel not in model["operations"]:
            return []

        operation_def = model["operations"][operation_camel]
        if "input" not in operation_def:
            return []

        input_shape_name = operation_def["input"].get("shape")
        if not input_shape_name or "shapes" not in model:
            return []

        input_shape = model["shapes"].get(input_shape_name, {})
        members = input_shape.get("members", {})

        # Convert member names to CLI parameter format (--parameter-name)
        parameters = []
        for member_name in members.keys():
            param_name = self._camel_to_kebab(member_name)
            parameters.append(f"--{param_name}")

        return sorted(parameters)

    def get_service_metadata(self, service: str) -> Optional[dict]:
        """Get service metadata (name, description, etc.)."""
        model = self._load_service_model(service)
        if not model:
            return None

        return model.get("metadata", {})


# Global singleton instance
_loader_instance: Optional[ServiceModelLoader] = None


def get_service_loader() -> ServiceModelLoader:
    """Get or create the global ServiceModelLoader instance."""
    global _loader_instance
    if _loader_instance is None:
        _loader_instance = ServiceModelLoader()
    return _loader_instance



"""Main Textual application and entry point."""

import sys
import os
import subprocess
from argparse import ArgumentParser
from threading import Event
from typing import List
from time import time

from textual.app import App, ComposeResult
from textual.screen import Screen
from textual.widgets import (
    Header,
    Footer,
    Input,
    ListView,
    ListItem,
    Static,
    RichLog,
)
from textual.containers import Container, Vertical, Horizontal, VerticalScroll
from textual.binding import Binding
from textual.reactive import reactive
from textual.worker import Worker
from textual.timer import Timer
from textual import events, work
from rich.console import Group
from rich.table import Table

from .config import parse_profiles
from .models import Profile
from .aws_cli import check_aws_cli_available, ensure_authenticated, sso_login
from .logging import get_logger
from .autocomplete import CommandAutocomplete
from .q_assistant import (
    check_q_cli_available,
    query_q_cli,
    stream_q_cli_query,
    clean_ansi_codes,
    format_aws_context,
)
from .cheatsheet import AWS_CLI_CHEATSHEET, AWS_CLI_COMMANDS, COMMAND_CATEGORIES
from .i18n import LANG_ZH_TW, LANG_EN


def parse_args():
    """Parse command-line arguments."""
    parser = ArgumentParser(prog="awsui", description="AWS Profile/SSO switcher TUI")

    parser.add_argument("--profile", help="Pre-select profile by name")
    parser.add_argument("--region", help="Override AWS region")
    parser.add_argument(
        "--lang",
        choices=["zh-TW", "en"],
        default="en",
        help="UI language (default: en)",
    )
    parser.add_argument(
        "--log-level",
        choices=["DEBUG", "INFO", "WARNING", "ERROR"],
        default="INFO",
        help="Logging level (default: INFO)",
    )

    return parser.parse_args()


class CheatsheetScreen(Screen):
    """Modal screen for displaying AWS CLI cheatsheet."""

    CSS = """
    CheatsheetScreen {
        layout: vertical;
        background: rgba(0, 0, 0, 0.75);
        align: center middle;
    }

    #cheatsheet-container {
        width: 90%;
        max-width: 160;
        height: 90%;
        background: $surface;
        border: thick $primary;
        padding: 2;
        layout: vertical;
        align-horizontal: left;
        align-vertical: top;
    }

    #cheatsheet-title {
        text-align: center;
    }

    #cheatsheet-content {
        height: 1fr;
        width: 100%;
    }

    #cheatsheet-body {
        width: 100%;
    }
    """

    BINDINGS = [
        ("escape", "close", "Close"),
        ("q", "close", "Close"),
    ]

    def compose(self) -> ComposeResult:
        """Compose the cheatsheet modal."""
        with Container(id="cheatsheet-container"):
            title = self.app.lang["cheatsheet_title"]
            dismiss = self.app.lang["cheatsheet_dismiss"]
            yield Static(
                f"[bold cyan]{title}[/bold cyan]\n[dim]{dismiss}[/dim]",
                id="cheatsheet-title",
            )
            with VerticalScroll(id="cheatsheet-content"):
                content = ""
                for category, commands in AWS_CLI_CHEATSHEET.items():
                    content += f"\n[bold yellow]{category}:[/bold yellow]\n"
                    for cmd in commands:
                        content += f"  [green]{cmd}[/green]\n"
                yield Static(content, id="cheatsheet-body")

    def action_close(self) -> None:
        """Close the cheatsheet and return to main screen."""
        self.app.pop_screen()


class RegionInputScreen(Screen):
    """Modal screen for region override input."""

    CSS = """
    RegionInputScreen {
        layout: vertical;
        background: rgba(0, 0, 0, 0.75);
        align: center middle;
    }

    #region-container {
        width: 60;
        height: auto;
        background: $surface;
        border: thick $primary;
        padding: 2;
        layout: vertical;
    }

    #region-title {
        text-align: center;
        margin-bottom: 1;
    }

    #region-hint {
        color: $text-muted;
        text-align: center;
        margin-bottom: 1;
    }

    #region-input {
        margin-bottom: 1;
    }
    """

    BINDINGS = [
        ("escape", "cancel", "Cancel"),
    ]

    def compose(self) -> ComposeResult:
        """Compose the region input modal."""
        with Container(id="region-container"):
            yield Static(
                f"[bold cyan]{self.app.lang['region_input_title']}[/bold cyan]",
                id="region-title",
            )
            yield Static(
                f"[dim]{self.app.lang['region_input_hint']}[/dim]",
                id="region-hint",
            )
            yield Input(
                placeholder=self.app.lang["region_input_placeholder"],
                id="region-input",
            )

    def on_mount(self) -> None:
        """Focus input when screen is mounted."""
        self.query_one("#region-input", Input).focus()

    def on_input_submitted(self, event: Input.Submitted) -> None:
        """Handle input submission."""
        region = event.value.strip()
        self.dismiss(region if region else None)

    def action_cancel(self) -> None:
        """Cancel and close the dialog."""
        self.dismiss(None)


class ProfileList(ListView):
    """Custom ListView for profiles with filtering."""

    def __init__(self, profiles: List[Profile], *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.all_profiles = profiles
        self.filtered_profiles = profiles.copy()

    def filter_profiles(self, query: str):
        """Filter profiles by search query."""
        if not query:
            self.filtered_profiles = self.all_profiles.copy()
        else:
            query_lower = query.lower()
            self.filtered_profiles = [
                p
                for p in self.all_profiles
                if query_lower in p["name"].lower()
                or (p["account"] and query_lower in p["account"])
                or (p["role"] and query_lower in p["role"].lower())
                or (p["region"] and query_lower in p["region"].lower())
            ]

        self.refresh_items()

    def refresh_items(self):
        """Refresh list items based on filtered profiles."""
        self.clear()
        for profile in self.filtered_profiles:
            title = f"[b]{profile['name']}[/b]"
            meta_parts: list[str] = []

            kind = profile.get("kind")
            if kind:
                meta_parts.append(kind.upper())
            account = profile.get("account")
            if account:
                meta_parts.append(str(account))
            role = profile.get("role")
            if role:
                meta_parts.append(role)
            region = profile.get("region")
            if region:
                meta_parts.append(region)

            body_text = title
            if meta_parts:
                meta_line = " | ".join(meta_parts)
                body_text += f"\n[dim]{meta_line}[/dim]"

            item_body = Static(body_text, markup=True, expand=True)
            item = ListItem(item_body, name=profile["name"])
            item.add_class("profile-list-item")
            self.append(item)


class AWSUIApp(App):
    MOUSE = False
    TITLE = "AWSUI"
    """Textual TUI application for AWS profile switching."""

    CSS = """
    Screen {
        layout: vertical;
        background: #0f172a;
        color: #e2e8f0;
    }

    Header {
        background: #0b1628;
        color: #f8fafc;
        text-style: bold;
    }

    Footer {
        background: #0b1628;
        color: #94a3b8;
    }

    #content {
        layout: horizontal;
        height: 1fr;
        padding: 0;
    }

    #left-pane {
        width: 30%;
        min-width: 25;
        max-width: 45;
        padding: 1 2 1 2;
        background: #0b1420;
        border-right: solid #1e293b;
    }

    #right-pane {
        layout: vertical;
        width: 1fr;
        padding: 0 1 0 1;
        background: #0f172a;
    }

    .section-title {
        text-style: bold;
        color: #38bdf8;
        background: #0d1926;
        padding: 0 1;
        margin-bottom: 1;
    }

    .section-spacing {
        margin-top: 1;
    }

    .section-hint {
        color: #64748b;
        text-style: italic;
        padding: 0 1;
        margin-bottom: 1;
    }

    #search {
        height: 3;
        margin-bottom: 1;
        border: tall $primary-darken-2;
        background: #0d1a2a;
    }

    #search:focus {
        border: tall $primary;
        background: #102339;
    }

    #profile-list {
        height: 1fr;
        width: 1fr;
        overflow-y: auto;
        border: solid #1e293b;
        background: #0a1018;
    }

    #profile-list > .list-item {
        margin: 0 1 1 1;
        padding: 1;
        border: solid transparent;
        background: transparent;
    }

    #profile-list > .list-item:last-child {
        margin-bottom: 0;
    }

    #profile-list > .list-item--highlight {
        border: solid $primary;
        background: $primary 20%;
    }

    #detail-pane {
        margin-top: 1;
        padding: 0;
        width: 1fr;
        min-height: 8;
        max-height: 14;
        overflow-y: auto;
        overflow-x: auto;
        scrollbar-size: 1 1;
        border: solid #1f2d44;
        background: #15263f;
    }

    #detail-content {
        padding: 1;
        color: #e2e8f0;
        text-align: left;
    }

    #ai-status {
        height: auto;
        margin: 0 0 0 0;
    }

    /* Output area (shared by CLI and AI) */
    #output-area {
        height: 1fr;
        padding: 0 1;
        margin-bottom: 1;
        background: #0a1018;
        border: solid #1e293b;
    }

    /* Input container */
    #input-container {
        layout: vertical;
        height: auto;
    }

    #autocomplete {
        display: none;
        max-height: 10;
        padding: 0;
        margin-bottom: 1;
        background: #0f1f33;
        border: solid $warning-darken-1;
    }

    #autocomplete > .list-item {
        padding: 0 1;
    }

    #autocomplete > .list-item--highlight {
        background: $warning 30%;
    }

    #shared-input {
        height: 3;
        border: tall $primary-darken-2;
        background: #0d1a2a;
    }

    #shared-input:focus {
        border: tall $primary;
        background: #102339;
    }

    .input-cli-mode {
        border: tall $success-darken-2;
    }

    .input-cli-mode:focus {
        border: tall $success;
    }

    .input-ai-mode {
        border: tall $warning-darken-2;
    }

    .input-ai-mode:focus {
        border: tall $warning;
    }

    Toast {
        width: 26;
        min-width: 20;
        padding: 0 1;
    }

    Toast .toast--title {
        text-style: bold;
    }

    .empty-state {
        width: 100%;
        content-align: center middle;
        text-align: center;
        color: #64748b;
        padding: 2;
    }

    .status-indicator {
        color: #38bdf8;
        padding: 0 1;
    }
    """

    BINDINGS = [
        Binding("/", "focus_search", "Search", show=True),
        Binding("c", "focus_cli", "CLI", show=True),
        Binding("a", "toggle_ai_panel", "AI", show=True),
        Binding("t", "toggle_left_pane", "Toggle", show=True),
        Binding("h", "show_cheatsheet", "Cheat", show=True),
        Binding("enter", "apply_profile", "Apply", show=True),
        Binding("l", "force_login", "Login", show=True),
        Binding("ctrl+c", "cancel_login", "Cancel", show=True),
        Binding("w", "whoami", "WhoAmI", show=True),
        Binding("r", "region_override", "Region", show=True),
        Binding("ctrl+l", "clear_cli", "Clear", show=False),
        Binding("escape", "blur_input", "Esc", show=True),
        Binding("?", "show_help", "Help", show=True),
        Binding("q", "quit", "Quit", show=True),
    ]

    show_left_pane: reactive[bool] = reactive(True)
    active_tab: reactive[str] = reactive("cli")

    def __init__(
        self,
        lang: str = "en",
        profile: str | None = None,
        region: str | None = None,
        log_level: str = "INFO",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.lang = LANG_ZH_TW if lang == "zh-TW" else LANG_EN
        self.sub_title = self.lang["app_subtitle"]
        self.preselect_profile = profile
        self.override_region = region
        self.logger = get_logger(log_level)
        self.profiles: List[Profile] = []
        self.selected_profile: Profile | None = None

        self.q_available: bool = False

        self.ai_spinner_frames: list[str] = ["⏳", "⌛", "🔄"]
        self.ai_spinner_index: int = 0
        self.ai_spinner_message: str = ""
        self.ai_spinner_timer: Timer | None = None
        self.ai_status_clear_timer: Timer | None = None

        self.command_history: List[str] = []
        self.history_index: int = -1
        self.current_input: str = ""
        self.browsing_history: bool = False
        self._autocomplete_handled_enter: bool = False

        self.auth_worker: Worker | None = None
        self.auth_worker_active: bool = False
        self.auth_cancel_event: Event | None = None
        self.authenticating_profile: Profile | None = None
        self.authenticating_fetch_identity: bool = False

    def compose(self) -> ComposeResult:
        """Compose the application layout."""
        yield Header(show_clock=True)

        with Horizontal(id="content"):
            with Vertical(id="left-pane"):
                yield Static(self.lang["panel_profiles"], classes="section-title")

                yield Input(placeholder=self.lang["search_placeholder"], id="search")

                yield ProfileList(self.profiles, id="profile-list")

                yield Static(
                    self.lang["panel_detail"],
                    classes="section-title section-spacing",
                )
                detail_placeholder = Static(
                    self.lang["detail_placeholder"],
                    id="detail-content",
                )
                detail_placeholder.add_class("empty-state")
                with VerticalScroll(id="detail-pane"):
                    yield detail_placeholder

            with Vertical(id="right-pane"):
                yield RichLog(id="output-area", highlight=True, markup=True)

                with Vertical(id="input-container"):
                    yield CommandAutocomplete(
                        AWS_CLI_COMMANDS, COMMAND_CATEGORIES, id="autocomplete"
                    )

                    ai_status = Static("", id="ai-status", classes="status-indicator")
                    ai_status.display = False
                    yield ai_status

                    yield Input(
                        placeholder=self.lang["cli_placeholder"],
                        id="shared-input",
                        classes="input-cli-mode",
                    )

        yield Footer()

    def on_mount(self):
        """Initialize app after mounting."""
        start_time = time()

        self.q_available = check_q_cli_available()
        self.logger.info("q_cli_check", available=self.q_available)

        self.active_tab = "cli"

        if not check_aws_cli_available():
            self.update_status(self.lang["no_aws_cli"], error=True)
            self.show_empty_state(self.lang["no_aws_cli"], self.lang["no_aws_cli_hint"])
            return

        self.profiles = parse_profiles()
        self.logger.info("profiles_loaded", count=len(self.profiles))

        if not self.profiles:
            self.update_status(self.lang["no_profiles"], error=True)
            self.show_empty_state(
                self.lang["no_profiles"], self.lang["no_profiles_hint"]
            )
            return

        profile_list = self.query_one("#profile-list", ProfileList)
        profile_list.all_profiles = self.profiles
        profile_list.filtered_profiles = self.profiles.copy()
        profile_list.refresh_items()

        if self.preselect_profile:
            for i, p in enumerate(self.profiles):
                if p["name"] == self.preselect_profile:
                    profile_list.index = i
                    self.selected_profile = p
                    self.update_detail_pane(p)
                    break

        duration_ms = int((time() - start_time) * 1000)
        self.logger.info("app_started", duration_ms=duration_ms)
        self.update_status(
            self.lang["profiles_loaded"].format(count=len(self.profiles))
        )

    def show_empty_state(self, title: str, hint: str):
        """Show empty state message."""
        detail_content = self.query_one("#detail-content", Static)
        detail_content.update(f"[bold]{title}[/bold]\n\n{hint}")
        detail_content.add_class("empty-state")

    def update_status(self, message: str, error: bool = False):
        """Display status notification."""
        if error:
            self.notify(message, severity="error", title=self.lang.get("error_title", "Error"))
        else:
            self.notify(message, severity="information")

    def _start_ai_spinner(self, message: str) -> None:
        """Begin animated status indicator for Amazon Q queries."""
        if self.ai_spinner_timer:
            self.ai_spinner_timer.stop()
            self.ai_spinner_timer = None
        if self.ai_status_clear_timer:
            self.ai_status_clear_timer.stop()
            self.ai_status_clear_timer = None

        self.ai_spinner_message = message
        self.ai_spinner_index = 0

        ai_status = self.query_one("#ai-status", Static)
        ai_status.display = True
        frame = self.ai_spinner_frames[self.ai_spinner_index]
        ai_status.update(f"[dim]{frame} {message}[/dim]")

        def tick() -> None:
            self.ai_spinner_index = (self.ai_spinner_index + 1) % len(
                self.ai_spinner_frames
            )
            current = self.ai_spinner_frames[self.ai_spinner_index]
            ai_status.update(f"[dim]{current} {self.ai_spinner_message}[/dim]")

        self.ai_spinner_timer = self.set_interval(0.45, tick)

    def _clear_ai_status(self) -> None:
        """Clear the Amazon Q status indicator."""
        ai_status = self.query_one("#ai-status", Static)
        ai_status.update("")
        ai_status.display = False

    def _stop_ai_spinner(self, message: str | None, success: bool = True) -> None:
        """Stop spinner and optionally show final message."""
        if self.ai_spinner_timer:
            self.ai_spinner_timer.stop()
            self.ai_spinner_timer = None

        if self.ai_status_clear_timer:
            self.ai_status_clear_timer.stop()
            self.ai_status_clear_timer = None

        if message:
            icon = "✅" if success else "⚠️"
            ai_status = self.query_one("#ai-status", Static)
            ai_status.display = True
            ai_status.update(f"{icon} {message}")
            self.ai_status_clear_timer = self.set_timer(3.0, self._clear_ai_status)
        else:
            self._clear_ai_status()

    def build_profile_detail(self, profile: Profile) -> Table:
        """Create a Rich table summarizing profile information."""
        table = Table.grid(padding=(0, 1))
        table.expand = True
        table.pad_edge = False
        table.add_column(justify="left", style="bold cyan", no_wrap=True)
        table.add_column(justify="left", no_wrap=True)

        def fmt(value: str | None, transform=None) -> str:
            if value is None or value == "":
                return "-"
            if transform:
                return transform(value)
            return str(value)

        table.add_row(self.lang["detail_name"], fmt(profile.get("name")))
        table.add_row(
            self.lang["detail_kind"],
            fmt(profile.get("kind"), lambda value: str(value).upper()),
        )
        table.add_row(self.lang["detail_account"], fmt(profile.get("account")))
        table.add_row(self.lang["detail_role"], fmt(profile.get("role")))

        # Show region with override indicator if applicable
        if self.override_region:
            region_label = self.lang["detail_region_override"]
            region_value = self.override_region
        else:
            region_label = self.lang["detail_region"]
            region_value = fmt(profile.get("region"))
        table.add_row(region_label, region_value)

        table.add_row(self.lang["detail_session"], fmt(profile.get("session")))

        return table

    def build_identity_detail(self, identity: dict) -> Table:
        """Create a Rich table summarizing identity details."""
        table = Table.grid(padding=(0, 1))
        table.expand = True
        table.pad_edge = False
        table.add_column(justify="left", style="bold yellow", no_wrap=True)
        table.add_column(justify="left", no_wrap=True)

        table.add_row(f"[b]{self.lang['whoami']}[/b]", "")
        table.add_row(self.lang["whoami_account"], identity.get("Account") or "-")
        table.add_row(self.lang["whoami_arn"], identity.get("Arn") or "-")
        table.add_row(self.lang["whoami_user"], identity.get("UserId") or "-")

        return table

    def update_detail_pane(self, profile: Profile):
        """Update detail pane with profile information."""
        detail_content = self.query_one("#detail-content", Static)
        detail_content.remove_class("empty-state")
        detail_content.update(self.build_profile_detail(profile))

    def on_key(self, event: events.Key) -> None:
        """Handle global key events when Input has focus."""
        focused = self.focused
        if not isinstance(focused, Input):
            return

        if focused.id == "shared-input":
            autocomplete = self.query_one("#autocomplete", CommandAutocomplete)

            if event.key == "ctrl+u":
                focused.value = ""
                self.browsing_history = False
                self.history_index = -1
                autocomplete.display = False
                autocomplete.clear_options()
                event.prevent_default()
                event.stop()
                return

            # Case 1: Browsing history → ↑↓ continues history navigation
            if self.browsing_history:
                if event.key == "up":
                    self.navigate_history_up(focused)
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key == "down":
                    self.navigate_history_down(focused)
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key not in ["up", "down"]:
                    self.browsing_history = False

            # Case 2: Input is empty → ↑↓ starts history navigation
            input_is_empty = not focused.value.strip()
            if input_is_empty:
                if event.key == "up":
                    self.navigate_history_up(focused)
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key == "down":
                    self.navigate_history_down(focused)
                    event.prevent_default()
                    event.stop()
                    return

            # Case 3: Input has content + autocomplete visible → ↑↓ for autocomplete, Tab/Enter to select
            elif autocomplete.display:
                if event.key == "up":
                    autocomplete.move_cursor_up()
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key == "down":
                    autocomplete.move_cursor_down()
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key == "tab":
                    selected = autocomplete.get_selected_command()
                    if selected:
                        current_value = focused.value
                        cursor_pos = focused.cursor_position

                        new_value, new_cursor_pos = autocomplete.smart_insert_selection(
                            current_value, cursor_pos, selected
                        )

                        focused.value = new_value
                        focused.cursor_position = new_cursor_pos
                        autocomplete.display = False
                        autocomplete.clear_options()
                        event.prevent_default()
                        event.stop()
                        return
                elif event.key == "enter":
                    selected = autocomplete.get_selected_command()
                    if selected:
                        current_value = focused.value
                        cursor_pos = focused.cursor_position

                        new_value, new_cursor_pos = autocomplete.smart_insert_selection(
                            current_value, cursor_pos, selected
                        )

                        focused.value = new_value
                        focused.cursor_position = new_cursor_pos
                        autocomplete.display = False
                        autocomplete.clear_options()
                        self._autocomplete_handled_enter = True
                        event.prevent_default()
                        event.stop()
                        return
                elif event.key == "escape":
                    autocomplete.display = False
                    autocomplete.clear_options()
                    event.prevent_default()
                    event.stop()
                    return

            # Case 4: Input has content but no autocomplete → ↑↓ for history navigation
            else:
                if event.key == "up":
                    self.navigate_history_up(focused)
                    event.prevent_default()
                    event.stop()
                    return
                elif event.key == "down":
                    self.navigate_history_down(focused)
                    event.prevent_default()
                    event.stop()
                    return

        if event.key == "escape":
            profile_list = self.query_one("#profile-list", ProfileList)
            profile_list.focus()
            event.prevent_default()
            event.stop()
            return

        if event.key == "q":
            self.action_quit()
            event.prevent_default()
            event.stop()
            return

        if event.key == "question_mark":
            self.action_show_help()
            event.prevent_default()
            event.stop()
            return

        if event.key == "ctrl+l":
            self.action_clear_cli()
            event.prevent_default()
            event.stop()
            return

    def on_input_changed(self, event: Input.Changed):
        """Handle search input changes and CLI autocomplete."""
        if event.input.id == "search":
            profile_list = self.query_one("#profile-list", ProfileList)
            profile_list.filter_profiles(event.value)
        elif event.input.id == "shared-input":
            autocomplete = self.query_one("#autocomplete", CommandAutocomplete)

            if self.browsing_history:
                if self.history_index != -1 and self.history_index < len(
                    self.command_history
                ):
                    expected_value = self.command_history[self.history_index]
                    if event.value != expected_value:
                        self.browsing_history = False
                        self.history_index = -1
                        self.current_input = ""
                        cursor_pos = event.input.cursor_position
                        autocomplete.filter_commands(event.value, cursor_pos)
                return

            if self.active_tab == "cli":
                cursor_pos = event.input.cursor_position
                autocomplete.filter_commands(event.value, cursor_pos)
            else:
                autocomplete.display = False

    def on_input_submitted(self, event: Input.Submitted):
        """Handle Enter key in search box and CLI input."""
        if event.input.id == "search":
            profile_list = self.query_one("#profile-list", ProfileList)

            # Check if there are filtered results
            if profile_list.filtered_profiles:
                # If only one result, apply it directly
                if len(profile_list.filtered_profiles) == 1:
                    profile_list.index = 0
                    self.selected_profile = profile_list.filtered_profiles[0]
                    self.update_detail_pane(self.selected_profile)
                    # Directly apply the single result
                    self.action_apply_profile()
                else:
                    # Multiple results: select first and move focus to list
                    profile_list.index = 0
                    self.selected_profile = profile_list.filtered_profiles[0]
                    self.update_detail_pane(self.selected_profile)
                    profile_list.focus()
                    self.notify(
                        self.lang["search_first_result"].format(
                            count=len(profile_list.filtered_profiles)
                        )
                    )
            else:
                self.notify(
                    self.lang["search_no_results"], severity="warning"
                )

        elif event.input.id == "shared-input":
            # Check if autocomplete just handled this Enter key
            # (the flag is set in on_key before this event fires)
            if self._autocomplete_handled_enter:
                self._autocomplete_handled_enter = False
                return

            value = event.value.strip()
            if value:
                if self.active_tab == "cli":
                    self.execute_aws_command(value)
                elif self.active_tab == "ai":
                    self.execute_q_query(value)
                event.input.value = ""

    def on_list_view_selected(self, event: ListView.Selected):
        """Handle profile selection."""
        profile_list = self.query_one("#profile-list", ProfileList)
        selected_index = profile_list.index

        if 0 <= selected_index < len(profile_list.filtered_profiles):
            self.selected_profile = profile_list.filtered_profiles[selected_index]
            self.update_detail_pane(self.selected_profile)

    def action_focus_search(self):
        """Focus on search input."""
        search_input = self.query_one("#search", Input)
        search_input.focus()

    def action_blur_input(self):
        """Leave current Input and return to profile list."""
        profile_list = self.query_one("#profile-list", ProfileList)
        profile_list.focus()

    def action_toggle_left_pane(self):
        """Toggle visibility of left pane (profile list)."""
        self.show_left_pane = not self.show_left_pane

    def watch_show_left_pane(self, show: bool) -> None:
        """React to changes in left pane visibility."""
        try:
            left_pane = self.query_one("#left-pane", Vertical)
            left_pane.display = show

            if show:
                self.screen.remove_class("fullscreen")
                self.notify(self.lang["left_pane_shown"])
            else:
                self.screen.add_class("fullscreen")
                self.notify(self.lang["cli_fullscreen"])
        except Exception:
            pass

    def action_toggle_ai_panel(self):
        """Switch to AI tab."""
        if not self.q_available:
            self.update_status(self.lang["ai_not_available"], error=True)
            return
        self.active_tab = "ai"
        # Always focus input, even if already in AI mode
        try:
            shared_input = self.query_one("#shared-input", Input)
            shared_input.focus()
        except Exception:
            pass

    def action_focus_cli(self):
        """Switch to CLI tab and focus input."""
        self.active_tab = "cli"
        # Always focus input, even if already in CLI mode
        try:
            shared_input = self.query_one("#shared-input", Input)
            shared_input.focus()
        except Exception:
            pass

    def watch_active_tab(self, tab: str) -> None:
        """React to mode changes (CLI or AI)."""
        try:
            shared_input = self.query_one("#shared-input", Input)
            autocomplete = self.query_one("#autocomplete", CommandAutocomplete)

            if tab == "cli":
                shared_input.set_classes("input-cli-mode")
                shared_input.placeholder = self.lang["cli_placeholder"]
                autocomplete.display = False

                self.notify(self.lang["cli_mode"])
                shared_input.focus()

            elif tab == "ai":
                shared_input.set_classes("input-ai-mode")
                shared_input.placeholder = self.lang["ai_placeholder"]
                autocomplete.display = False
                autocomplete.clear_options()

                if not self.q_available:
                    output_area = self.query_one("#output-area", RichLog)
                    output_area.write(
                        f"\n[bold yellow]{self.lang['ai_not_available']}[/bold yellow]"
                    )
                    output_area.write(f"[dim]{self.lang['ai_install_hint']}[/dim]")
                    output_area.write(
                        "[cyan]https://docs.aws.amazon.com/amazonq/latest/qdeveloper-ug/command-line-installing.html[/cyan]\n"
                    )

                self.notify(self.lang["ai_mode"])
                shared_input.focus()

        except Exception:
            pass

    def action_apply_profile(self):
        """Apply the selected profile."""
        if not self.selected_profile:
            self.update_status(self.lang["select_profile_first"], error=True)
            return

        profile = self.selected_profile
        self.update_status(self.lang["authenticating"])
        self._start_login(profile, fetch_identity=True)

    def _start_login(self, profile: Profile, fetch_identity: bool) -> None:
        """Kick off background SSO login/authentication."""
        if self.auth_worker_active:
            self.update_status(self.lang["login_in_progress"])
            return

        cancel_event = Event()
        profile_snapshot = profile.copy()

        self.auth_worker_active = True
        self.auth_cancel_event = cancel_event
        self.authenticating_profile = profile_snapshot
        self.authenticating_fetch_identity = fetch_identity
        self.auth_worker = self.perform_login(
            profile_snapshot, fetch_identity, cancel_event
        )

    @work(exclusive=True, thread=True)
    def perform_login(
        self, profile: Profile, fetch_identity: bool, cancel_event: Event
    ):
        """Run authentication or login logic in a worker thread."""
        profile_name = profile["name"]
        start_time = time()
        identity = None
        success = False

        if fetch_identity:
            identity = ensure_authenticated(
                profile_name, cancel_check=cancel_event.is_set
            )
            success = identity is not None
        else:
            success = sso_login(profile_name, cancel_check=cancel_event.is_set)

        duration_ms = int((time() - start_time) * 1000)
        cancelled = cancel_event.is_set()

        self.call_from_thread(
            self._finish_login,
            profile,
            identity,
            success,
            duration_ms,
            cancelled,
            fetch_identity,
        )

    def _finish_login(
        self,
        profile: Profile,
        identity: dict | None,
        success: bool,
        duration_ms: int,
        cancelled: bool,
        fetch_identity: bool,
    ) -> None:
        """Handle completion of background login/authentication."""
        profile_name = profile["name"]

        self.auth_worker = None
        self.auth_worker_active = False
        self.auth_cancel_event = None
        self.authenticating_profile = None
        self.authenticating_fetch_identity = False

        if cancelled:
            if fetch_identity:
                self.logger.info(
                    "authentication_cancelled",
                    profile=profile_name,
                    duration_ms=duration_ms,
                )
                self.update_status(self.lang["auth_cancelled"])
            else:
                self.logger.info(
                    "sso_login_cancelled", profile=profile_name, duration_ms=duration_ms
                )
                self.update_status(self.lang["login_cancelled"])
            return

        if fetch_identity:
            if success and identity:
                self.logger.info(
                    "authentication_success",
                    profile=profile_name,
                    duration_ms=duration_ms,
                    account=identity.get("Account"),
                    arn=identity.get("Arn"),
                )
                self.update_status(self.lang["auth_success"])

                # Update detail pane with profile info
                if (
                    self.selected_profile
                    and self.selected_profile["name"] == profile_name
                ):
                    detail_content = self.query_one("#detail-content", Static)
                    detail_content.remove_class("empty-state")
                    renderables = [self.build_profile_detail(profile)]
                    if identity:
                        renderables.append(self.build_identity_detail(identity))

                    detail_content.update(
                        Group(*renderables)
                        if len(renderables) > 1
                        else renderables[0]
                    )
            else:
                self.logger.error(
                    "authentication_failed",
                    profile=profile_name,
                    duration_ms=duration_ms,
                )
                self.update_status(self.lang["auth_failed"], error=True)
        else:
            if success:
                self.logger.info(
                    "sso_login_success", profile=profile_name, duration_ms=duration_ms
                )
                self.update_status(self.lang["login_success"])
            else:
                self.logger.error(
                    "sso_login_failed", profile=profile_name, duration_ms=duration_ms
                )
                self.update_status(self.lang["login_failed"], error=True)

    def action_force_login(self):
        """Force SSO login for selected profile."""
        if not self.selected_profile:
            self.update_status(self.lang["select_profile_first"], error=True)
            return

        profile = self.selected_profile
        profile_name = profile["name"]
        self.update_status(self.lang["login_loading"].format(profile=profile_name))
        self._start_login(profile, fetch_identity=False)

    def action_cancel_login(self):
        """Cancel an in-progress SSO login or authentication."""
        if not self.auth_worker_active or not self.auth_cancel_event:
            self.update_status(self.lang["no_login_task"], error=True)
            return

        if not self.auth_cancel_event.is_set():
            self.auth_cancel_event.set()
            if self.authenticating_profile:
                profile_name = self.authenticating_profile.get("name", "")
                event_name = (
                    "authentication_cancel_requested"
                    if self.authenticating_fetch_identity
                    else "sso_login_cancel_requested"
                )
                if profile_name:
                    self.logger.info(event_name, profile=profile_name)
                else:
                    self.logger.info(event_name)
        self.update_status(self.lang["login_cancelling"])

    def action_whoami(self):
        """Trigger whoami identity check."""
        if not self.selected_profile:
            self.update_status(self.lang["select_profile_first"], error=True)
            return

        self.update_status(self.lang["whoami_checking"])
        # Make a copy to avoid thread safety issues
        profile_snapshot = self.selected_profile.copy()
        self.perform_whoami(profile_snapshot)

    @work(exclusive=False, thread=True)
    def perform_whoami(self, profile: Profile):
        """Execute whoami in worker thread."""
        from .aws_cli import get_caller_identity

        profile_name = profile["name"]
        identity = get_caller_identity(profile_name)

        # Update UI from thread
        self.call_from_thread(self._update_whoami_result, profile, identity)

    def _update_whoami_result(self, profile: Profile, identity: dict | None):
        """Update UI with whoami result (called from worker thread)."""
        if identity:
            # Only update if this profile is still selected
            if (
                self.selected_profile
                and self.selected_profile["name"] == profile["name"]
            ):
                detail_content = self.query_one("#detail-content", Static)
                detail_content.remove_class("empty-state")
                detail_content.update(self.build_identity_detail(identity))

                # Ensure profile list stays populated after UI update
                try:
                    profile_list = self.query_one("#profile-list", ProfileList)
                    # Check if list was accidentally cleared but has data
                    if (
                        profile_list.filtered_profiles
                        and len(list(profile_list.children)) == 0
                    ):
                        self.logger.warning(
                            "profile_list_empty_after_whoami",
                            filtered_count=len(profile_list.filtered_profiles),
                        )
                        profile_list.refresh_items()
                except Exception as e:
                    self.logger.error("profile_list_check_failed", error=str(e))

            self.update_status(self.lang["whoami_updated"])
        else:
            self.update_status(self.lang["whoami_failed"], error=True)

    def action_region_override(self):
        """Show region override dialog."""
        def handle_region_result(result: str | None) -> None:
            """Handle the result from region input screen."""
            if result is not None:
                # User entered empty string - clear override
                if result == "":
                    self.override_region = None
                    self.update_status(self.lang["region_override_cleared"])
                else:
                    # Set new override
                    self.override_region = result
                    self.update_status(self.lang["region_override_set"].format(region=result))

                # Update detail pane if a profile is selected
                if self.selected_profile:
                    detail_content = self.query_one("#detail-content", Static)
                    detail_content.update(self.build_profile_detail(self.selected_profile))

        self.push_screen(RegionInputScreen(), handle_region_result)

    def action_clear_cli(self):
        """Clear output area."""
        output_area = self.query_one("#output-area", RichLog)
        output_area.clear()
        self.notify(self.lang["output_cleared"])

    def navigate_history_up(self, input_widget: Input) -> None:
        """Navigate up in command history (older commands)."""
        if not self.command_history:
            return

        # First time pressing up: save current input
        if self.history_index == -1:
            self.current_input = input_widget.value
            self.history_index = len(self.command_history)

        # Move up
        if self.history_index > 0:
            self.history_index -= 1
            input_widget.value = self.command_history[self.history_index]
            input_widget.action_end()

            # Enter history browsing mode
            self.browsing_history = True

            # Close autocomplete
            autocomplete = self.query_one("#autocomplete", CommandAutocomplete)
            autocomplete.display = False
            autocomplete.clear_options()

    def navigate_history_down(self, input_widget: Input) -> None:
        """Navigate down in command history (newer commands)."""
        if self.history_index == -1:
            return  # Not in browsing mode

        # Move down
        self.history_index += 1

        if self.history_index >= len(self.command_history):
            # Return to original input
            input_widget.value = self.current_input
            self.history_index = -1
            self.browsing_history = False  # Exit history mode
        else:
            input_widget.value = self.command_history[self.history_index]
            input_widget.action_end()
            self.browsing_history = True  # Still in history mode

        # Close autocomplete
        autocomplete = self.query_one("#autocomplete", CommandAutocomplete)
        autocomplete.display = False
        autocomplete.clear_options()

    @work(exclusive=True, thread=True)
    def execute_aws_command(self, command: str):
        """Execute AWS CLI command and display output in real-time."""
        # Add to command history (avoid duplicates)
        if command and (
            not self.command_history or self.command_history[-1] != command
        ):
            self.command_history.append(command)

        # Reset history browsing state
        self.history_index = -1
        self.current_input = ""
        self.browsing_history = False

        output_area = self.query_one("#output-area", RichLog)

        # Show current profile and command
        profile_name = (
            self.selected_profile["name"]
            if self.selected_profile
            else self.lang["profile_none"]
        )
        self.call_from_thread(
            output_area.write, f"[bold cyan][{profile_name}] $ {command}[/bold cyan]"
        )

        # Setup environment with selected profile
        env = os.environ.copy()
        if self.selected_profile:
            env["AWS_PROFILE"] = self.selected_profile["name"]
            # Use override_region if set, otherwise use profile's region
            region = self.override_region or self.selected_profile.get("region")
            if region:
                env["AWS_DEFAULT_REGION"] = region

        # Execute command with real-time output streaming
        start_time = time()
        try:
            process = subprocess.Popen(
                command,
                shell=True,
                env=env,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                bufsize=1,  # Line buffered
            )

            # Stream stdout in real-time
            if process.stdout:
                for line in iter(process.stdout.readline, ""):
                    if line:
                        self.call_from_thread(output_area.write, line.rstrip())

            # Wait for process to complete
            process.wait()

            # Get any remaining stderr
            if process.stderr:
                stderr_output = process.stderr.read()
                if stderr_output:
                    self.call_from_thread(
                        output_area.write, f"[red]{stderr_output.rstrip()}[/red]"
                    )

            duration_ms = int((time() - start_time) * 1000)

            # Display execution info
            if process.returncode == 0:
                self.call_from_thread(
                    output_area.write,
                    f"[dim]{self.lang['execute_success'].format(duration=duration_ms)}[/dim]\n",
                )
            else:
                self.call_from_thread(
                    output_area.write,
                    f"[red]{self.lang['cli_error_exit'].format(code=process.returncode, duration=duration_ms)}[/red]\n",
                )

            self.logger.info(
                "cli_command_executed",
                command=command,
                profile=profile_name,
                duration_ms=duration_ms,
                exit_code=process.returncode,
            )

        except Exception as e:
            self.call_from_thread(
                output_area.write,
                f"[red]{self.lang['cli_error_exception'].format(error=str(e))}[/red]\n",
            )
            self.logger.error("cli_command_error", command=command, error=str(e))

    @work(exclusive=False, thread=True)
    def execute_q_query(self, query: str):
        """Execute Amazon Q CLI query and display response."""
        output_area = self.query_one("#output-area", RichLog)

        # Show query
        self.call_from_thread(output_area.write, f"[bold cyan]Q: {query}[/bold cyan]")

        # Start animated status indicator
        self.call_from_thread(self._start_ai_spinner, self.lang["ai_spinner_wait"])

        # Prepare context
        context = ""
        if self.selected_profile:
            # Use override_region if set, otherwise use profile's region
            region = self.override_region or self.selected_profile.get("region")
            context = format_aws_context(
                profile_name=self.selected_profile.get("name"),
                region=region,
                account=self.selected_profile.get("account"),
            )

        # Show querying status
        self.call_from_thread(self.update_status, self.lang["ai_querying"])

        # Execute query with AWS profile environment (streaming)
        start_time = time()
        try:
            # Use override_region if set, otherwise use profile's region
            region = self.override_region or (
                self.selected_profile.get("region") if self.selected_profile else None
            )

            # Start streaming process
            process = stream_q_cli_query(
                prompt=query,
                context=context,
                profile_name=self.selected_profile.get("name")
                if self.selected_profile
                else None,
                region=region,
            )

            if not process:
                duration_ms = int((time() - start_time) * 1000)
                self.call_from_thread(
                    self._stop_ai_spinner, self.lang["ai_spinner_error"], False
                )
                self.call_from_thread(
                    output_area.write,
                    "[red]Amazon Q Developer CLI not available or failed to start.[/red]\n",
                )
                self.call_from_thread(
                    self.update_status, self.lang["ai_query_failed"], error=True
                )
                self.logger.error("q_query_failed", query=query, duration_ms=duration_ms)
                return

            # Stream output line by line
            output_lines = []
            success = True

            try:
                for line in process.stdout:
                    if line:
                        clean_line = clean_ansi_codes(line.rstrip('\n'))
                        output_lines.append(clean_line)
                        # Display line immediately
                        self.call_from_thread(output_area.write, f"[green]{clean_line}[/green]\n")

                # Wait for process to complete
                process.wait()

                # Check return code
                if process.returncode != 0:
                    success = False
                    stderr_output = process.stderr.read() if process.stderr else ""
                    if stderr_output:
                        clean_error = clean_ansi_codes(stderr_output.strip())
                        self.call_from_thread(
                            output_area.write, f"[red]Error: {clean_error}[/red]\n"
                        )

            except Exception as stream_exc:
                success = False
                self.call_from_thread(
                    output_area.write,
                    f"[red]Stream error: {str(stream_exc)}[/red]\n",
                )

        except Exception as exc:  # pragma: no cover - defensive
            duration_ms = int((time() - start_time) * 1000)
            error_text = str(exc) or "Unknown error"
            self.call_from_thread(
                self._stop_ai_spinner, self.lang["ai_spinner_error"], False
            )
            self.call_from_thread(
                output_area.write,
                f"[red]{self.lang['ai_error_exception'].format(error=error_text, duration=duration_ms)}[/red]\n",
            )
            self.call_from_thread(
                self.update_status, self.lang["ai_query_failed"], error=True
            )
            self.logger.exception(
                "q_query_exception", query=query, duration_ms=duration_ms
            )
            return

        duration_ms = int((time() - start_time) * 1000)

        # Update UI after completion
        if success:
            self.call_from_thread(
                self._stop_ai_spinner, self.lang["ai_spinner_done"], True
            )
            self.call_from_thread(
                output_area.write,
                f"[dim]{self.lang['execute_success'].format(duration=duration_ms)}[/dim]\n",
            )
            self.logger.info("q_query_success", query=query, duration_ms=duration_ms)
        else:
            self.call_from_thread(
                self._stop_ai_spinner, self.lang["ai_spinner_error"], False
            )
            self.call_from_thread(
                output_area.write,
                f"[dim]{self.lang['execute_failure'].format(duration=duration_ms)}[/dim]\n",
            )
            self.call_from_thread(
                self.update_status, self.lang["ai_query_failed"], error=True
            )
            response_text = "\n".join(output_lines) if output_lines else "Unknown error"
            self.logger.error(
                "q_query_failed", query=query, duration_ms=duration_ms, error=response_text
            )

    def action_show_cheatsheet(self):
        """Show AWS CLI cheatsheet in modal screen."""
        self.push_screen(CheatsheetScreen())

    def action_show_help(self):
        """Show help overlay."""
        detail_content = self.query_one("#detail-content", Static)
        detail_content.remove_class("empty-state")
        detail_content.update(self.lang["help_text"])
        self.notify(self.lang["help_displayed"])


def main():
    """Entry point for awsui CLI."""
    args = parse_args()

    # Create and run app
    app = AWSUIApp(
        lang=args.lang,
        profile=args.profile,
        region=args.region,
        log_level=args.log_level,
    )
    app.run()


if __name__ == "__main__":
    main()



"""Tests for special AWS CLI commands like help and configure."""

import pytest

from awsui.command_parser import AWSCommandParser, CompletionContext


@pytest.fixture()
def parser():
    parser = AWSCommandParser()
    parser.use_dynamic_loading = False
    return parser


def test_help_shortcut_suggestion(parser):
    parsed = parser.parse("aws h")
    assert parsed.current_context == CompletionContext.SERVICE
    assert parsed.current_token == "h"
    assert parser.get_suggestions(parsed) == ["help"]


def test_help_full_word(parser):
    parsed = parser.parse("aws help")
    assert parsed.service == "help"
    assert parsed.current_context == CompletionContext.COMMAND
    assert parsed.current_token == "help"


def test_configure_shortcut_precedes_services(parser):
    parsed = parser.parse("aws c")
    suggestions = parser.get_suggestions(parsed)
    assert suggestions[0] == "configure"
    assert "cloudformation" in suggestions


def test_special_commands_prioritized_at_root(parser):
    parsed = parser.parse("aws ")
    assert parsed.current_context == CompletionContext.SERVICE
    assert parser.get_suggestions(parsed)[:2] == AWSCommandParser.SPECIAL_COMMANDS



"""Custom autocomplete widget for AWS CLI commands."""

from textual.widgets import OptionList
from textual.widgets.option_list import Option
from textual.message import Message

from .command_parser import AWSCommandParser, CompletionContext
from .parameter_metadata import get_parameter_metadata, format_parameter_help
from .resource_suggester import ResourceSuggester


class CommandAutocomplete(OptionList):
    """Enhanced autocomplete with fuzzy matching and highlighting."""

    class CommandSelected(Message):
        """Message sent when a command is selected from autocomplete."""

        def __init__(self, command: str) -> None:
            self.command = command
            super().__init__()

    def __init__(self, commands: list[str], command_categories: dict[str, str], *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.all_commands = commands
        self.command_categories = command_categories
        self.filtered_commands: list[str] = []
        self.display = False
        self.can_focus = False

        self.parser = AWSCommandParser()
        self.use_intelligent_autocomplete = True
        self.resource_suggester: ResourceSuggester | None = None
        self.enable_resource_suggestions = True

    def set_aws_context(self, profile: str | None, region: str | None) -> None:
        """Set AWS profile and region for resource suggestions."""
        if self.enable_resource_suggestions:
            self.resource_suggester = ResourceSuggester(profile, region)

    def fuzzy_match(self, text: str, query: str) -> tuple[bool, int]:
        """
        Check if query fuzzy matches text with scoring.
        Returns (matched, score) where higher score = better match.
        """
        text_lower = text.lower()
        query_lower = query.lower()

        if query_lower in text_lower:
            position = text_lower.find(query_lower)
            score = 100 - (position * 2)
            return (True, max(score, 80))

        text_idx = 0
        query_idx = 0
        matches = 0

        while text_idx < len(text_lower) and query_idx < len(query_lower):
            if text_lower[text_idx] == query_lower[query_idx]:
                matches += 1
                query_idx += 1
            text_idx += 1

        if query_idx == len(query_lower):
            score = int((matches / len(text_lower)) * 60)
            return (True, max(score, 20))

        return (False, 0)

    def highlight_match(self, text: str, query: str) -> str:
        """Highlight matching substring in text."""
        if not query:
            return text

        lower_text = text.lower()
        lower_query = query.lower()
        start = lower_text.find(lower_query)

        if start >= 0:
            end = start + len(query)
            return f"{text[:start]}[bold yellow]{text[start:end]}[/]{text[end:]}"

        return text

    def filter_commands(self, query: str, cursor_pos: int | None = None) -> None:
        """
        Filter commands with intelligent or fuzzy matching.

        Args:
            query: The command line input
            cursor_pos: Cursor position in the input (for intelligent parsing)
        """
        if not query or len(query) < 2:
            self.display = False
            self.filtered_commands = []
            self.clear_options()
            return

        if self.use_intelligent_autocomplete and query.startswith("aws "):
            self._intelligent_filter(query, cursor_pos)
        else:
            self._fuzzy_filter(query)

    def _intelligent_filter(self, query: str, cursor_pos: int | None = None) -> None:
        """Use intelligent parser for context-aware suggestions."""
        parsed = self.parser.parse(query, cursor_pos)
        suggestions = self.parser.get_suggestions(parsed)

        if (parsed.current_context == CompletionContext.PARAMETER_VALUE and
                self.resource_suggester and parsed.service and parsed.command):
            last_param = None
            for param, value in reversed(list(parsed.parameters.items())):
                if value is None:
                    last_param = param
                    break

            if last_param:
                resource_suggestions = self.resource_suggester.get_suggestions_for_parameter(
                    parsed.service, parsed.command, last_param
                )
                if resource_suggestions:
                    query_lower = parsed.current_token.lower()
                    filtered_resources = [
                        r for r in resource_suggestions
                        if query_lower in r.lower()
                    ]
                    if filtered_resources:
                        suggestions = filtered_resources[:10]

        if suggestions:
            self.filtered_commands = suggestions[:10]

            self.clear_options()
            for suggestion in self.filtered_commands:
                if parsed.current_context == CompletionContext.SERVICE:
                    badge = "[dim cyan]SVC[/dim cyan]"
                    description = ""
                elif parsed.current_context == CompletionContext.COMMAND:
                    badge = "[dim green]CMD[/dim green]"
                    description = ""
                elif parsed.current_context == CompletionContext.PARAMETER:
                    badge = "[dim yellow]OPT[/dim yellow]"
                    metadata = get_parameter_metadata(parsed.service, suggestion)
                    if metadata:
                        required_mark = "*" if metadata.required else ""
                        description = f" [{metadata.param_type.value}]{required_mark} {metadata.description[:40]}"
                    else:
                        description = ""
                elif parsed.current_context == CompletionContext.PARAMETER_VALUE:
                    badge = "[dim magenta]VAL[/dim magenta]"
                    description = ""
                else:
                    badge = ""
                    description = ""

                highlighted = self.highlight_match(suggestion, parsed.current_token)
                label_text = f"{badge} {highlighted}{description}"
                self.add_option(Option(label_text, id=suggestion))

            self.display = True
            if len(self.filtered_commands) > 0:
                self.highlighted = 0
        else:
            self._fuzzy_filter(query)

    def _fuzzy_filter(self, query: str) -> None:
        """Legacy fuzzy matching filter (fallback)."""
        matches = []
        for cmd in self.all_commands:
            matched, score = self.fuzzy_match(cmd, query)
            if matched:
                matches.append((cmd, score))

        if matches:
            matches.sort(key=lambda x: x[1], reverse=True)
            self.filtered_commands = [cmd for cmd, _ in matches[:10]]

            self.clear_options()
            for cmd in self.filtered_commands:
                category = self.command_categories.get(cmd, "")
                if category:
                    badge_text = category.split("/")[0][:4].upper()
                    badge = f"[dim cyan]{badge_text}[/dim cyan]"
                else:
                    badge = ""

                highlighted = self.highlight_match(cmd, query)
                label_text = f"{badge} {highlighted}" if badge else highlighted
                self.add_option(Option(label_text, id=cmd))

            self.display = True
            if len(self.filtered_commands) > 0:
                self.highlighted = 0
        else:
            self.display = False
            self.filtered_commands = []
            self.clear_options()

    def get_selected_command(self) -> str | None:
        """Get currently highlighted command."""
        if self.highlighted is not None and 0 <= self.highlighted < len(self.filtered_commands):
            return self.filtered_commands[self.highlighted]
        return None

    def move_cursor_down(self) -> None:
        """Move selection down."""
        if self.filtered_commands and self.highlighted is not None:
            self.highlighted = min(self.highlighted + 1, len(self.filtered_commands) - 1)

    def move_cursor_up(self) -> None:
        """Move selection up."""
        if self.filtered_commands and self.highlighted is not None:
            self.highlighted = max(self.highlighted - 1, 0)

    def smart_insert_selection(
        self, current_value: str, cursor_pos: int, selection: str
    ) -> tuple[str, int]:
        """
        Intelligently insert the selected suggestion into the command line.

        Preserves all content before and after the current token being completed,
        replacing only the incomplete token with the selection.

        Args:
            current_value: Current command line text
            cursor_pos: Current cursor position
            selection: Selected suggestion to insert

        Returns:
            Tuple of (new_value, new_cursor_position)
        """
        if selection.startswith("aws "):
            return (selection, len(selection))

        if not self.use_intelligent_autocomplete or not current_value.strip().startswith("aws "):
            return (selection, len(selection))

        parsed = self.parser.parse(current_value, cursor_pos)
        token_start = cursor_pos - len(parsed.current_token)

        # Find where the current token actually ends (could be beyond cursor)
        token_end = cursor_pos
        while token_end < len(current_value) and not current_value[token_end].isspace():
            token_end += 1

        text_after_token = current_value[token_end:]

        if parsed.current_context in (
            CompletionContext.SERVICE,
            CompletionContext.COMMAND,
            CompletionContext.PARAMETER,
            CompletionContext.PARAMETER_VALUE,
        ):
            new_value = (
                current_value[:token_start] +
                selection +
                " " +
                text_after_token.lstrip()
            )
            new_cursor = token_start + len(selection) + 1
        else:
            if parsed.current_token:
                new_value = (
                    current_value[:token_start] +
                    selection +
                    " " +
                    text_after_token.lstrip()
                )
                new_cursor = token_start + len(selection) + 1
            else:
                new_value = current_value.rstrip() + " " + selection + " "
                new_cursor = len(new_value)

        return (new_value, new_cursor)


"""Data models and AWS config parsing."""

from typing import TypedDict, Literal


class Profile(TypedDict):
    """AWS Profile representation."""
    name: str
    kind: Literal["sso", "assume", "basic"]
    account: str | None
    role: str | None
    region: str | None
    session: str | None  # sso-session name
    source: str  # source file path


"""Dynamic AWS resource suggester for intelligent autocomplete."""

import subprocess
import json
from datetime import datetime, timedelta
from typing import Dict, List, Optional
import threading


class ResourceCache:
    """Thread-safe cache for AWS resource suggestions with TTL."""

    def __init__(self, ttl_seconds: int = 300):
        """
        Initialize cache.

        Args:
            ttl_seconds: Time-to-live for cached entries (default: 5 minutes)
        """
        self.ttl_seconds = ttl_seconds
        self._cache: Dict[str, tuple[datetime, List[str]]] = {}
        self._lock = threading.Lock()

    def get(self, key: str) -> Optional[List[str]]:
        """Get cached value if still valid."""
        with self._lock:
            if key in self._cache:
                timestamp, value = self._cache[key]
                if datetime.now() - timestamp < timedelta(seconds=self.ttl_seconds):
                    return value
                else:
                    del self._cache[key]
        return None

    def set(self, key: str, value: List[str]) -> None:
        """Set cached value with current timestamp."""
        with self._lock:
            self._cache[key] = (datetime.now(), value)

    def clear(self) -> None:
        """Clear all cached entries."""
        with self._lock:
            self._cache.clear()


class ResourceSuggester:
    """Suggests AWS resource values by querying AWS CLI."""

    def __init__(self, profile: Optional[str] = None, region: Optional[str] = None):
        """
        Initialize resource suggester.

        Args:
            profile: AWS profile to use for queries
            region: AWS region to use for queries
        """
        self.profile = profile
        self.region = region
        self.cache = ResourceCache(ttl_seconds=300)  # 5 minute cache

    def _run_aws_command(self, command: List[str], timeout: int = 10) -> Optional[str]:
        """
        Run AWS CLI command and return output.

        Args:
            command: AWS CLI command as list of arguments
            timeout: Command timeout in seconds

        Returns:
            Command output or None if failed
        """
        try:
            if self.profile:
                command.extend(["--profile", self.profile])
            if self.region:
                command.extend(["--region", self.region])

            command.extend(["--output", "json"])

            result = subprocess.run(
                command,
                capture_output=True,
                text=True,
                timeout=timeout,
                check=False,
            )

            if result.returncode == 0:
                return result.stdout
            else:
                return None

        except (subprocess.TimeoutExpired, Exception):
            return None

    def get_ec2_instance_ids(self) -> List[str]:
        """Get list of EC2 instance IDs."""
        cache_key = f"ec2:instances:{self.profile}:{self.region}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "ec2", "describe-instances", "--query", "Reservations[].Instances[].InstanceId"]
        )

        if output:
            try:
                instance_ids = json.loads(output)
                self.cache.set(cache_key, instance_ids)
                return instance_ids
            except json.JSONDecodeError:
                pass

        return []

    def get_s3_buckets(self) -> List[str]:
        """Get list of S3 bucket names."""
        cache_key = f"s3:buckets:{self.profile}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "s3api", "list-buckets", "--query", "Buckets[].Name"]
        )

        if output:
            try:
                buckets = json.loads(output)
                self.cache.set(cache_key, buckets)
                return buckets
            except json.JSONDecodeError:
                pass

        return []

    def get_lambda_functions(self) -> List[str]:
        """Get list of Lambda function names."""
        cache_key = f"lambda:functions:{self.profile}:{self.region}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "lambda", "list-functions", "--query", "Functions[].FunctionName"]
        )

        if output:
            try:
                functions = json.loads(output)
                self.cache.set(cache_key, functions)
                return functions
            except json.JSONDecodeError:
                pass

        return []

    def get_dynamodb_tables(self) -> List[str]:
        """Get list of DynamoDB table names."""
        cache_key = f"dynamodb:tables:{self.profile}:{self.region}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "dynamodb", "list-tables", "--query", "TableNames"]
        )

        if output:
            try:
                tables = json.loads(output)
                self.cache.set(cache_key, tables)
                return tables
            except json.JSONDecodeError:
                pass

        return []

    def get_iam_roles(self) -> List[str]:
        """Get list of IAM role names."""
        cache_key = f"iam:roles:{self.profile}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "iam", "list-roles", "--query", "Roles[].RoleName"]
        )

        if output:
            try:
                roles = json.loads(output)
                self.cache.set(cache_key, roles)
                return roles
            except json.JSONDecodeError:
                pass

        return []

    def get_security_groups(self) -> List[str]:
        """Get list of security group IDs."""
        cache_key = f"ec2:security-groups:{self.profile}:{self.region}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "ec2", "describe-security-groups", "--query", "SecurityGroups[].GroupId"]
        )

        if output:
            try:
                sgs = json.loads(output)
                self.cache.set(cache_key, sgs)
                return sgs
            except json.JSONDecodeError:
                pass

        return []

    def get_vpcs(self) -> List[str]:
        """Get list of VPC IDs."""
        cache_key = f"ec2:vpcs:{self.profile}:{self.region}"
        cached = self.cache.get(cache_key)
        if cached is not None:
            return cached

        output = self._run_aws_command(
            ["aws", "ec2", "describe-vpcs", "--query", "Vpcs[].VpcId"]
        )

        if output:
            try:
                vpcs = json.loads(output)
                self.cache.set(cache_key, vpcs)
                return vpcs
            except json.JSONDecodeError:
                pass

        return []

    def get_suggestions_for_parameter(
        self, service: str, command: str, parameter: str
    ) -> Optional[List[str]]:
        """
        Get resource suggestions for a specific parameter.

        Args:
            service: AWS service (e.g., "ec2", "s3")
            command: Service command
            parameter: Parameter name (e.g., "--instance-ids")

        Returns:
            List of suggested values or None if not applicable
        """
        param_map = {
            "--instance-ids": self.get_ec2_instance_ids,
            "--instance-id": self.get_ec2_instance_ids,
            "--function-name": self.get_lambda_functions,
            "--table-name": self.get_dynamodb_tables,
            "--role-name": self.get_iam_roles,
            "--security-group-ids": self.get_security_groups,
            "--vpc-id": self.get_vpcs,
        }

        if service == "s3" and command in ["cp", "ls", "sync", "rm", "rb"]:
            return self.get_s3_buckets()

        if parameter in param_map:
            return param_map[parameter]()

        return None



"""Tests for global parameters support."""

import pytest
from awsui.command_parser import AWSCommandParser, CompletionContext


@pytest.fixture
def parser():
    parser = AWSCommandParser()
    parser.use_dynamic_loading = False
    return parser


def test_global_parameters_at_root(parser):
    """Test global parameters shown when typing 'aws --'."""
    parsed = parser.parse("aws --")
    assert parsed.current_context == CompletionContext.PARAMETER
    suggestions = parser.get_suggestions(parsed)
    assert "--version" in suggestions
    assert "--debug" in suggestions
    assert "--region" in suggestions
    assert "--profile" in suggestions


def test_global_parameter_filter(parser):
    """Test filtering global parameters with prefix."""
    parsed = parser.parse("aws --v")
    suggestions = parser.get_suggestions(parsed)
    assert "--version" in suggestions
    assert "--debug" not in suggestions


def test_global_parameter_with_service(parser):
    """Test global parameters included with service parameters."""
    parsed = parser.parse("aws s3 ls --")
    suggestions = parser.get_suggestions(parsed)
    assert "--region" in suggestions
    assert "--output" in suggestions
    assert "--recursive" in suggestions


def test_global_parameter_before_service(parser):
    """Test global parameters can appear before service."""
    parsed = parser.parse("aws --region ")
    assert parsed.current_context == CompletionContext.PARAMETER_VALUE
    suggestions = parser.get_suggestions(parsed)
    assert "us-east-1" in suggestions
    assert "us-west-2" in suggestions


def test_no_parameters_without_dash(parser):
    """Test parameters not shown when not typing dash."""
    parsed = parser.parse("aws s")
    suggestions = parser.get_suggestions(parsed)
    assert "s3" in suggestions
    assert "--version" not in suggestions
    assert "--region" not in suggestions


def test_global_parameters_list_complete(parser):
    """Test all expected global parameters are defined."""
    expected_params = [
        "--version",
        "--debug",
        "--no-verify-ssl",
        "--no-paginate",
        "--output",
        "--query",
        "--profile",
        "--region",
        "--endpoint-url",
        "--no-cli-pager",
    ]
    for param in expected_params:
        assert param in parser.GLOBAL_PARAMETERS



"""Amazon Q Developer CLI integration for AI-powered assistance."""

import os
import re
import shutil
import subprocess
from typing import Callable


def check_q_cli_available() -> bool:
    """
    Check if Amazon Q Developer CLI is available.

    Returns True if 'q' command is found in PATH, False otherwise.
    """
    return shutil.which("q") is not None


def get_q_cli_version() -> str | None:
    """
    Get Amazon Q Developer CLI version.

    Returns version string or None if unable to determine.
    """
    try:
        result = subprocess.run(
            ["q", "--version"],
            capture_output=True,
            text=True,
            timeout=5
        )
        if result.returncode == 0:
            return result.stdout.strip()
        return None
    except (subprocess.SubprocessError, FileNotFoundError):
        return None


def query_q_cli(
    prompt: str,
    context: str | None = None,
    profile_name: str | None = None,
    region: str | None = None,
    cancel_check: Callable[[], bool] | None = None,
    timeout: int = 300
) -> tuple[str, bool]:
    """
    Query Amazon Q Developer CLI with a prompt.

    Args:
        prompt: User's question or command to ask Q
        context: Optional context information (profile, region, etc.)
        profile_name: AWS profile name to use
        region: AWS region to use
        cancel_check: Optional callable that returns True if operation should be cancelled
        timeout: Timeout in seconds (default: 300)

    Returns:
        Tuple of (response_text, success)
        - response_text: Q's response or error message
        - success: True if query succeeded, False otherwise
    """
    if not check_q_cli_available():
        return ("Amazon Q Developer CLI not available. Please install it first.", False)

    full_prompt = prompt
    if context:
        full_prompt = f"{context}\n\n{prompt}"

    env = os.environ.copy()
    if profile_name:
        env["AWS_PROFILE"] = profile_name
    if region:
        env["AWS_DEFAULT_REGION"] = region

    try:
        # Note: --trust-all-tools allows Q to use tools without confirmation
        process = subprocess.Popen(
            ["q", "chat", "--no-interactive", "--trust-all-tools", "--wrap", "never", full_prompt],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            env=env
        )

        if cancel_check and cancel_check():
            process.terminate()
            try:
                process.wait(timeout=2)
            except subprocess.TimeoutExpired:
                process.kill()
            return ("Query cancelled.", False)

        try:
            stdout, stderr = process.communicate(timeout=timeout)
        except subprocess.TimeoutExpired:
            process.kill()
            stdout, stderr = process.communicate()
            return ("Query timed out after {timeout} seconds.", False)

        if process.returncode == 0:
            clean_output = clean_ansi_codes(stdout.strip())
            return (clean_output, True)
        else:
            error_msg = stderr.strip() if stderr else "Unknown error occurred"
            clean_error = clean_ansi_codes(error_msg)
            return (f"Error: {clean_error}", False)

    except FileNotFoundError:
        return ("Amazon Q Developer CLI 'q' command not found.", False)
    except Exception as e:
        return (f"Unexpected error: {str(e)}", False)


def stream_q_cli_query(
    prompt: str,
    context: str | None = None,
    profile_name: str | None = None,
    region: str | None = None,
    cancel_check: Callable[[], bool] | None = None,
    timeout: int = 300
) -> subprocess.Popen[str] | None:
    """
    Start a streaming query to Amazon Q Developer CLI.

    Args:
        prompt: User's question or command to ask Q
        context: Optional context information (profile, region, etc.)
        profile_name: AWS profile name to use
        region: AWS region to use
        cancel_check: Optional callable that returns True if operation should be cancelled
        timeout: Timeout in seconds (default: 300)

    Returns:
        subprocess.Popen object with stdout available for streaming, or None on failure
        Caller is responsible for handling the process and reading from stdout
    """
    if not check_q_cli_available():
        return None

    if cancel_check and cancel_check():
        return None

    full_prompt = prompt
    if context:
        full_prompt = f"{context}\n\n{prompt}"

    env = os.environ.copy()
    if profile_name:
        env["AWS_PROFILE"] = profile_name
    if region:
        env["AWS_DEFAULT_REGION"] = region

    try:
        # Note: --trust-all-tools allows Q to use tools without confirmation
        process = subprocess.Popen(
            ["q", "chat", "--no-interactive", "--trust-all-tools", "--wrap", "never", full_prompt],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            bufsize=1,  # Line buffered
            env=env
        )
        return process
    except (FileNotFoundError, subprocess.SubprocessError):
        return None


def clean_ansi_codes(text: str) -> str:
    """
    Remove ANSI escape codes from text.

    Args:
        text: Text potentially containing ANSI codes

    Returns:
        Clean text without ANSI codes
    """
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    return ansi_escape.sub('', text)


def format_aws_context(
    profile_name: str | None = None,
    region: str | None = None,
    account: str | None = None
) -> str:
    """
    Format AWS context information for Q CLI query.

    Args:
        profile_name: Current AWS profile name
        region: Current AWS region
        account: Current AWS account ID

    Returns:
        Formatted context string
    """
    context_parts = []

    if profile_name:
        context_parts.append(f"Current AWS Profile: {profile_name}")
    if region:
        context_parts.append(f"Region: {region}")
    if account:
        context_parts.append(f"Account ID: {account}")

    if context_parts:
        return "Context: " + ", ".join(context_parts)
    return ""


"""Internationalization (i18n) - UI translations for awsui."""

LANG_ZH_TW = {
    "search_placeholder": "搜尋 profiles (按 / 聚焦)...",
    "cli_placeholder": "輸入 AWS CLI 指令（開始輸入顯示建議，空白時 ↑↓ 瀏覽歷史）",
    "no_profiles": "未偵測到 profiles",
    "no_profiles_hint": "請執行 'aws configure sso-session' 或 'aws configure sso' 建立",
    "no_aws_cli": "未偵測到 AWS CLI v2",
    "no_aws_cli_hint": "請依官方文件安裝 AWS CLI v2",
    "detail_name": "名稱",
    "detail_kind": "類型",
    "detail_account": "帳號",
    "detail_role": "角色",
    "detail_region": "區域",
    "detail_session": "SSO Session",
    "panel_profiles": "Profiles",
    "panel_profiles_help": "搜尋與切換",
    "panel_detail": "Profile 詳情",
    "detail_placeholder": "選擇 profile 以顯示資料",
    "panel_cli": "CLI 終端",
    "panel_cli_help": "輸出與歷史",
    "panel_cli_input": "指令輸入",
    "app_subtitle": "Profile 切換與 CLI 助手",
    "authenticating": "驗證中...",
    "login_required": "需要登入",
    "login_success": "登入成功",
    "login_failed": "登入失敗",
    "login_cancelling": "取消登入中...",
    "login_cancelled": "登入已取消",
    "login_in_progress": "登入處理中，請稍候",
    "auth_success": "驗證成功",
    "auth_failed": "驗證失敗",
    "auth_cancelled": "驗證已取消",
    "whoami": "當前身份",
    "whoami_updated": "身份資訊已更新",
    "whoami_failed": "無法取得身份資訊",
    "whoami_account": "帳號",
    "whoami_arn": "ARN",
    "whoami_user": "使用者 ID",
    "no_login_task": "目前沒有登入作業",
    "select_profile_first": "請先選擇 profile",
    "panel_ai": "AI 助手",
    "panel_ai_help": "Amazon Q 開發助手",
    "ai_placeholder": "詢問 Amazon Q (例: 如何列出所有 S3 buckets?)",
    "ai_not_available": "Amazon Q CLI 不可用",
    "ai_install_hint": "請先安裝 Amazon Q Developer CLI",
    "ai_spinner_wait": "正在查詢 Amazon Q",
    "ai_spinner_done": "查詢完成",
    "ai_spinner_error": "查詢失敗",
    "ai_querying": "查詢中...",
    "ai_query_failed": "查詢失敗",
    "ai_cancelled": "已取消查詢",
    "whoami_checking": "正在取得身份資訊...",
    "search_first_result": "已選擇第一個結果，共 {count} 個符合",
    "search_no_results": "沒有符合的 profiles",
    "left_pane_shown": "左側面板已顯示",
    "cli_fullscreen": "CLI 滿版模式 - 按 t 恢復",
    "cli_mode": "CLI 模式",
    "ai_mode": "AI 模式",
    "output_cleared": "輸出已清空",
    "help_displayed": "說明已顯示",
    "region_override_wip": "Region override - 功能開發中",
    "region_input_title": "覆寫區域",
    "region_input_placeholder": "輸入 AWS 區域（例: us-west-2）",
    "region_input_hint": "留空使用 profile 預設值",
    "region_override_set": "區域已覆寫為：{region}",
    "region_override_cleared": "區域覆寫已清除",
    "detail_region_override": "區域（已覆寫）",
    "login_loading": "登入 {profile}...",
    "profiles_loaded": "載入 {count} 個 profiles",
    "execute_success": "✓ 完成 ({duration}ms)",
    "execute_failure": "✗ 失敗 ({duration}ms)",
    "cli_error_exit": "✗ 錯誤 (exit code: {code}, {duration}ms)",
    "ai_error_exception": "✗ 發生錯誤: {error} ({duration}ms)",
    "cli_error_exception": "✗ 執行錯誤: {error}",
    "profile_none": "未選擇",
    "error_title": "錯誤",
    "cheatsheet_title": "AWS CLI Cheatsheet",
    "cheatsheet_dismiss": "按 Esc 或 q 關閉",
    "help_text": """[bold]快捷鍵說明:[/bold]

/ - 聚焦搜尋框
c - 切換到 CLI 模式
a - 切換到 AI 助手模式
t - 切換左側面板顯示/隱藏
h - 顯示 AWS CLI Cheatsheet
Enter - 套用選定的 profile
l - 強制執行 SSO login
w - 顯示當前身份 (WhoAmI)
Ctrl+L - 清空輸出區域
Ctrl+U - 清空輸入框
Esc - 離開輸入框
? - 顯示此說明
q - 離開程式

[bold]CLI 輸入框智慧導航:[/bold]

空白時：
  ↑↓ - 瀏覽歷史指令

瀏覽歷史時：
  ↑↓ - 繼續瀏覽（不會觸發 autocomplete）
  修改內容 - 自動離開歷史模式

輸入內容後有建議時：
  ↑↓ - 在 autocomplete 建議中選擇
  Enter - 確認選擇

輸入內容但沒建議時：
  ↑↓ - 瀏覽歷史指令

[bold]使用方式:[/bold]

1. 使用搜尋框過濾 profiles
2. 上下鍵選擇 profile
3. 按 c 進入 CLI 模式，或按 a 進入 AI 助手模式
4. CLI 模式：空白時按 ↑↓ 快速找歷史指令
5. CLI 模式：開始輸入，自動顯示建議（↑↓ 選擇）
6. AI 模式：直接輸入自然語言問題
7. Ctrl+U 快速清空輸入
8. 按 h 查看常用 AWS CLI 指令
9. 按 t 可隱藏左側面板，讓輸出區域滿版顯示""",
}

LANG_EN = {
    "search_placeholder": "Search profiles (press / to focus)...",
    "cli_placeholder": "AWS CLI command (type to see suggestions, use ↑↓ for history)",
    "no_profiles": "No profiles detected",
    "no_profiles_hint": "Please run 'aws configure sso-session' or 'aws configure sso'",
    "no_aws_cli": "AWS CLI v2 not detected",
    "no_aws_cli_hint": "Please install AWS CLI v2 per official documentation",
    "detail_name": "Name",
    "detail_kind": "Type",
    "detail_account": "Account",
    "detail_role": "Role",
    "detail_region": "Region",
    "detail_session": "SSO Session",
    "panel_profiles": "Profiles",
    "panel_profiles_help": "Search & switch",
    "panel_detail": "Profile Details",
    "detail_placeholder": "Select a profile to view details",
    "panel_cli": "Command Console",
    "panel_cli_help": "Output & history",
    "panel_cli_input": "Command Input",
    "app_subtitle": "Profile switcher & CLI helper",
    "authenticating": "Authenticating...",
    "login_required": "Login required",
    "login_success": "Login successful",
    "login_failed": "Login failed",
    "login_cancelling": "Cancelling login...",
    "login_cancelled": "Login cancelled",
    "login_in_progress": "Login already in progress",
    "auth_success": "Authentication successful",
    "auth_failed": "Authentication failed",
    "auth_cancelled": "Authentication cancelled",
    "whoami": "Current Identity",
    "whoami_updated": "Identity refreshed",
    "whoami_failed": "Unable to fetch identity",
    "whoami_account": "Account",
    "whoami_arn": "ARN",
    "whoami_user": "UserId",
    "no_login_task": "No login is currently running",
    "select_profile_first": "Select a profile first",
    "panel_ai": "AI Assistant",
    "panel_ai_help": "Amazon Q Developer",
    "ai_placeholder": "Ask Amazon Q (e.g., How to list all S3 buckets?)",
    "ai_not_available": "Amazon Q CLI not available",
    "ai_install_hint": "Please install Amazon Q Developer CLI first",
    "ai_spinner_wait": "Querying Amazon Q",
    "ai_spinner_done": "Query complete",
    "ai_spinner_error": "Query failed",
    "ai_querying": "Querying...",
    "ai_query_failed": "Query failed",
    "ai_cancelled": "Query cancelled",
    "whoami_checking": "Fetching identity...",
    "search_first_result": "Selected first result; {count} matches",
    "search_no_results": "No matching profiles",
    "left_pane_shown": "Left pane shown",
    "cli_fullscreen": "CLI fullscreen mode – press t to restore",
    "cli_mode": "Switched to CLI mode",
    "ai_mode": "Switched to AI mode",
    "output_cleared": "Output cleared",
    "help_displayed": "Help displayed",
    "region_override_wip": "Region override – coming soon",
    "region_input_title": "Override Region",
    "region_input_placeholder": "Enter AWS region (e.g., us-west-2)",
    "region_input_hint": "Leave empty to use profile default",
    "region_override_set": "Region override set to: {region}",
    "region_override_cleared": "Region override cleared",
    "detail_region_override": "Region (Override)",
    "login_loading": "Logging in to {profile}...",
    "profiles_loaded": "Loaded {count} profiles",
    "execute_success": "✓ Completed ({duration}ms)",
    "execute_failure": "✗ Failed ({duration}ms)",
    "cli_error_exit": "✗ Error (exit code: {code}, {duration}ms)",
    "ai_error_exception": "✗ Error: {error} ({duration}ms)",
    "cli_error_exception": "✗ Execution error: {error}",
    "profile_none": "No profile",
    "error_title": "Error",
    "cheatsheet_title": "AWS CLI Cheatsheet",
    "cheatsheet_dismiss": "Press Esc or q to close",
    "help_text": """[bold]Keyboard Shortcuts:[/bold]

/ - Focus search box
c - Switch to CLI mode
a - Switch to AI assistant mode
t - Toggle left pane visibility
h - Show AWS CLI Cheatsheet
Enter - Apply selected profile
l - Force SSO login
w - Show current identity (WhoAmI)
Ctrl+L - Clear output area
Ctrl+U - Clear input field
Esc - Exit input field
? - Show this help
q - Quit

[bold]CLI Input Smart Navigation:[/bold]

When empty:
  ↑↓ - Browse command history

While browsing history:
  ↑↓ - Continue browsing (won't trigger autocomplete)
  Type - Exit history mode

When input has content with suggestions:
  ↑↓ - Select from autocomplete suggestions
  Enter - Confirm selection

When input has content without suggestions:
  ↑↓ - Browse command history

[bold]Quick Start:[/bold]

1. Use search box to filter profiles
2. Use ↑↓ keys to select profile
3. Press c for CLI mode, or press a for AI mode
4. CLI mode: Press ↑↓ when empty to browse history
5. CLI mode: Start typing to see suggestions (↑↓ to select)
6. AI mode: Enter natural language questions
7. Press Ctrl+U to quickly clear input
8. Press h to view common AWS CLI commands
9. Press t to hide left pane for fullscreen output""",
}



"""AWS CLI command cheatsheet for quick reference."""

AWS_CLI_CHEATSHEET = {
    "CloudFormation": [
        "aws cloudformation list-stacks",
        "aws cloudformation describe-stacks --stack-name <name>",
        "aws cloudformation create-stack --stack-name <name> --template-body file://template.yml",
        "aws cloudformation update-stack --stack-name <name> --template-body file://template.yml",
        "aws cloudformation delete-stack --stack-name <name>",
    ],
    "CloudFront": [
        "aws cloudfront list-distributions",
        "aws cloudfront create-invalidation --distribution-id <id> --paths '/*'",
        "aws cloudfront get-distribution-config --id <id>",
    ],
    "CloudTrail": [
        "aws cloudtrail describe-trails",
        "aws cloudtrail lookup-events --max-results 20",
        "aws cloudtrail start-logging --name <trail-name>",
    ],
    "CloudWatch Logs": [
        "aws logs tail /aws/lambda/function-name --follow",
        "aws logs describe-log-groups",
        "aws logs describe-log-streams --log-group-name <name>",
    ],
    "CloudWatch Metrics": [
        "aws cloudwatch list-metrics --namespace AWS/EC2",
        "aws cloudwatch get-metric-statistics --namespace AWS/EC2 --metric-name CPUUtilization --dimensions Name=InstanceId,Value=<id> --start-time <iso> --end-time <iso> --period 300 --statistics Average",
        "aws cloudwatch describe-alarms",
    ],
    "DynamoDB": [
        "aws dynamodb list-tables",
        "aws dynamodb describe-table --table-name <name>",
        "aws dynamodb scan --table-name <name>",
        "aws dynamodb update-table --table-name <name> --provisioned-throughput ReadCapacityUnits=5,WriteCapacityUnits=5",
    ],
    "EC2": [
        "aws ec2 describe-instances",
        "aws ec2 start-instances --instance-ids i-xxxxx",
        "aws ec2 stop-instances --instance-ids i-xxxxx",
        "aws ec2 reboot-instances --instance-ids i-xxxxx",
        "aws ec2 describe-security-groups",
        "aws ec2 describe-vpcs",
        "aws ec2 describe-subnets",
        "aws ec2 create-image --instance-id i-xxxxx --name <image-name>",
    ],
    "ECR": [
        "aws ecr describe-repositories",
        "aws ecr get-login-password | docker login --username AWS --password-stdin <account>.dkr.ecr.<region>.amazonaws.com",
        "aws ecr list-images --repository-name <repo>",
        "aws ecr batch-delete-image --repository-name <repo> --image-ids imageTag=<tag>",
    ],
    "ECS": [
        "aws ecs list-clusters",
        "aws ecs list-services --cluster <cluster>",
        "aws ecs describe-services --cluster <cluster> --services <service>",
        "aws ecs update-service --cluster <cluster> --service <service> --force-new-deployment",
    ],
    "EKS": [
        "aws eks list-clusters",
        "aws eks describe-cluster --name <cluster>",
        "aws eks update-kubeconfig --name <cluster>",
    ],
    "Glue": [
        "aws glue get-databases",
        "aws glue get-tables --database-name <db>",
        "aws glue start-job-run --job-name <job>",
    ],
    "IAM": [
        "aws iam list-users",
        "aws iam list-roles",
        "aws iam get-user",
        "aws iam list-policies",
        "aws iam attach-role-policy --role-name <role> --policy-arn <arn>",
    ],
    "Identity/STS": [
        "aws sts get-caller-identity",
        "aws sts assume-role --role-arn <arn> --role-session-name <name>",
        "aws sts get-session-token",
    ],
    "Kinesis": [
        "aws kinesis list-streams",
        "aws kinesis describe-stream --stream-name <name>",
        "aws kinesis get-shard-iterator --stream-name <name> --shard-id <id> --shard-iterator-type TRIM_HORIZON",
    ],
    "KMS": [
        "aws kms list-keys",
        "aws kms describe-key --key-id <key-id>",
        "aws kms encrypt --key-id <key-id> --plaintext fileb://data.txt --output text --query CiphertextBlob",
    ],
    "Lambda": [
        "aws lambda list-functions",
        "aws lambda invoke --function-name <name> output.txt",
        "aws lambda get-function --function-name <name>",
        "aws lambda update-function-code --function-name <name> --zip-file fileb://function.zip",
    ],
    "Organizations": [
        "aws organizations list-accounts",
        "aws organizations describe-organization",
        "aws organizations list-parents --child-id <account-id>",
    ],
    "RDS": [
        "aws rds describe-db-instances",
        "aws rds describe-db-clusters",
        "aws rds create-db-snapshot --db-instance-identifier <id> --db-snapshot-identifier <snapshot>",
    ],
    "Redshift": [
        "aws redshift describe-clusters",
        "aws redshift describe-snapshot-schedules",
        "aws redshift create-cluster-snapshot --cluster-identifier <id> --snapshot-identifier <snapshot>",
    ],
    "Route53": [
        "aws route53 list-hosted-zones",
        "aws route53 list-resource-record-sets --hosted-zone-id <id>",
        "aws route53 change-resource-record-sets --hosted-zone-id <id> --change-batch file://change.json",
    ],
    "S3": [
        "aws s3 ls",
        "aws s3 ls s3://bucket-name",
        "aws s3 cp file.txt s3://bucket/",
        "aws s3 cp s3://bucket/file.txt .",
        "aws s3 sync ./dir s3://bucket/",
        "aws s3 mb s3://bucket-name",
        "aws s3 rb s3://bucket-name --force",
        "aws s3 rm s3://bucket/file.txt",
    ],
    "Secrets Manager": [
        "aws secretsmanager list-secrets",
        "aws secretsmanager get-secret-value --secret-id <name>",
        "aws secretsmanager update-secret --secret-id <name> --secret-string '<json>'",
    ],
    "SNS": [
        "aws sns list-topics",
        "aws sns publish --topic-arn <arn> --message \"Hello\"",
        "aws sns subscribe --topic-arn <arn> --protocol email --notification-endpoint <email>",
    ],
    "SQS": [
        "aws sqs list-queues",
        "aws sqs send-message --queue-url <url> --message-body 'Hello'",
        "aws sqs receive-message --queue-url <url>",
        "aws sqs purge-queue --queue-url <url>",
    ],
    "SSM": [
        "aws ssm describe-instance-information",
        "aws ssm get-parameter --name <name> --with-decryption",
        "aws ssm start-session --target <instance-id>",
        "aws ssm send-command --document-name AWS-RunShellScript --targets Key=instanceids,Values=<id> --parameters commands=['uptime']",
    ],
    "Step Functions": [
        "aws stepfunctions list-state-machines",
        "aws stepfunctions describe-state-machine --state-machine-arn <arn>",
        "aws stepfunctions start-execution --state-machine-arn <arn> --input file://input.json",
    ],
}

AWS_CLI_COMMANDS = [cmd for cmds in AWS_CLI_CHEATSHEET.values() for cmd in cmds]

COMMAND_CATEGORIES = {}
for category, commands in AWS_CLI_CHEATSHEET.items():
    for cmd in commands:
        COMMAND_CATEGORIES[cmd] = category



"""AWS CLI parameter metadata for enhanced autocomplete."""

from dataclasses import dataclass
from enum import Enum


class ParameterType(Enum):
    """Parameter value types."""
    STRING = "string"
    INTEGER = "integer"
    BOOLEAN = "boolean"
    LIST = "list"
    JSON = "json"
    FILE = "file"


@dataclass
class ParameterMetadata:
    """Metadata for an AWS CLI parameter."""
    name: str
    description: str
    required: bool = False
    param_type: ParameterType = ParameterType.STRING
    example: str = ""


# Common AWS CLI parameters with metadata
COMMON_PARAMETER_METADATA = {
    "--region": ParameterMetadata(
        name="--region",
        description="AWS region to use for this command",
        required=False,
        param_type=ParameterType.STRING,
        example="us-east-1"
    ),
    "--output": ParameterMetadata(
        name="--output",
        description="Output format (json, yaml, text, table)",
        required=False,
        param_type=ParameterType.STRING,
        example="json"
    ),
    "--profile": ParameterMetadata(
        name="--profile",
        description="AWS credential profile to use",
        required=False,
        param_type=ParameterType.STRING,
        example="default"
    ),
    "--query": ParameterMetadata(
        name="--query",
        description="JMESPath query to filter output",
        required=False,
        param_type=ParameterType.STRING,
        example="Reservations[].Instances[].InstanceId"
    ),
    "--no-cli-pager": ParameterMetadata(
        name="--no-cli-pager",
        description="Disable AWS CLI pager",
        required=False,
        param_type=ParameterType.BOOLEAN,
        example=""
    ),
    "--max-items": ParameterMetadata(
        name="--max-items",
        description="Maximum number of items to return",
        required=False,
        param_type=ParameterType.INTEGER,
        example="100"
    ),
}

# Service-specific parameter metadata
SERVICE_PARAMETER_METADATA = {
    "s3": {
        "--recursive": ParameterMetadata(
            name="--recursive",
            description="Recursively process files and subdirectories",
            required=False,
            param_type=ParameterType.BOOLEAN
        ),
        "--exclude": ParameterMetadata(
            name="--exclude",
            description="Exclude files matching pattern",
            required=False,
            param_type=ParameterType.STRING,
            example="*.log"
        ),
        "--include": ParameterMetadata(
            name="--include",
            description="Include only files matching pattern",
            required=False,
            param_type=ParameterType.STRING,
            example="*.txt"
        ),
        "--delete": ParameterMetadata(
            name="--delete",
            description="Delete files in destination not in source",
            required=False,
            param_type=ParameterType.BOOLEAN
        ),
        "--acl": ParameterMetadata(
            name="--acl",
            description="Access control list (private, public-read, etc.)",
            required=False,
            param_type=ParameterType.STRING,
            example="private"
        ),
    },
    "ec2": {
        "--instance-ids": ParameterMetadata(
            name="--instance-ids",
            description="EC2 instance IDs to operate on",
            required=True,
            param_type=ParameterType.LIST,
            example="i-1234567890abcdef0"
        ),
        "--security-group-ids": ParameterMetadata(
            name="--security-group-ids",
            description="Security group IDs",
            required=False,
            param_type=ParameterType.LIST,
            example="sg-12345678"
        ),
        "--vpc-id": ParameterMetadata(
            name="--vpc-id",
            description="VPC ID",
            required=False,
            param_type=ParameterType.STRING,
            example="vpc-12345678"
        ),
        "--subnet-id": ParameterMetadata(
            name="--subnet-id",
            description="Subnet ID",
            required=False,
            param_type=ParameterType.STRING,
            example="subnet-12345678"
        ),
        "--filters": ParameterMetadata(
            name="--filters",
            description="Filter results by criteria",
            required=False,
            param_type=ParameterType.JSON,
            example='Name=instance-state-name,Values=running'
        ),
    },
    "lambda": {
        "--function-name": ParameterMetadata(
            name="--function-name",
            description="Lambda function name or ARN",
            required=True,
            param_type=ParameterType.STRING,
            example="my-function"
        ),
        "--payload": ParameterMetadata(
            name="--payload",
            description="JSON payload to pass to function",
            required=False,
            param_type=ParameterType.JSON,
            example='{"key":"value"}'
        ),
        "--zip-file": ParameterMetadata(
            name="--zip-file",
            description="Deployment package (ZIP file)",
            required=False,
            param_type=ParameterType.FILE,
            example="fileb://function.zip"
        ),
        "--runtime": ParameterMetadata(
            name="--runtime",
            description="Runtime environment (python3.11, nodejs18.x, etc.)",
            required=False,
            param_type=ParameterType.STRING,
            example="python3.11"
        ),
        "--handler": ParameterMetadata(
            name="--handler",
            description="Function handler (file.method)",
            required=False,
            param_type=ParameterType.STRING,
            example="index.handler"
        ),
        "--role": ParameterMetadata(
            name="--role",
            description="IAM role ARN for function execution",
            required=True,
            param_type=ParameterType.STRING,
            example="arn:aws:iam::123456789012:role/lambda-role"
        ),
    },
    "iam": {
        "--role-name": ParameterMetadata(
            name="--role-name",
            description="IAM role name",
            required=True,
            param_type=ParameterType.STRING,
            example="MyRole"
        ),
        "--policy-arn": ParameterMetadata(
            name="--policy-arn",
            description="IAM policy ARN",
            required=True,
            param_type=ParameterType.STRING,
            example="arn:aws:iam::aws:policy/ReadOnlyAccess"
        ),
        "--user-name": ParameterMetadata(
            name="--user-name",
            description="IAM user name",
            required=True,
            param_type=ParameterType.STRING,
            example="john.doe"
        ),
        "--group-name": ParameterMetadata(
            name="--group-name",
            description="IAM group name",
            required=True,
            param_type=ParameterType.STRING,
            example="Developers"
        ),
    },
    "dynamodb": {
        "--table-name": ParameterMetadata(
            name="--table-name",
            description="DynamoDB table name",
            required=True,
            param_type=ParameterType.STRING,
            example="MyTable"
        ),
        "--key": ParameterMetadata(
            name="--key",
            description="Primary key of the item",
            required=False,
            param_type=ParameterType.JSON,
            example='{"id":{"S":"123"}}'
        ),
        "--attribute-definitions": ParameterMetadata(
            name="--attribute-definitions",
            description="Attribute definitions for table schema",
            required=False,
            param_type=ParameterType.JSON,
            example='AttributeName=id,AttributeType=S'
        ),
    },
    "cloudformation": {
        "--stack-name": ParameterMetadata(
            name="--stack-name",
            description="CloudFormation stack name",
            required=True,
            param_type=ParameterType.STRING,
            example="my-stack"
        ),
        "--template-body": ParameterMetadata(
            name="--template-body",
            description="CloudFormation template file",
            required=False,
            param_type=ParameterType.FILE,
            example="file://template.yaml"
        ),
        "--template-url": ParameterMetadata(
            name="--template-url",
            description="S3 URL to CloudFormation template",
            required=False,
            param_type=ParameterType.STRING,
            example="https://s3.amazonaws.com/bucket/template.yaml"
        ),
        "--parameters": ParameterMetadata(
            name="--parameters",
            description="Stack parameters",
            required=False,
            param_type=ParameterType.JSON,
            example="ParameterKey=KeyName,ParameterValue=MyKey"
        ),
    },
}


def get_parameter_metadata(service: str, parameter: str) -> ParameterMetadata | None:
    """
    Get metadata for a parameter.

    Args:
        service: AWS service name
        parameter: Parameter name (e.g., "--region")

    Returns:
        ParameterMetadata if found, None otherwise
    """
    # Check common parameters first
    if parameter in COMMON_PARAMETER_METADATA:
        return COMMON_PARAMETER_METADATA[parameter]

    # Check service-specific parameters
    if service in SERVICE_PARAMETER_METADATA:
        if parameter in SERVICE_PARAMETER_METADATA[service]:
            return SERVICE_PARAMETER_METADATA[service][parameter]

    return None


def format_parameter_help(metadata: ParameterMetadata) -> str:
    """
    Format parameter metadata as help text.

    Args:
        metadata: Parameter metadata

    Returns:
        Formatted help string
    """
    required_marker = " *" if metadata.required else ""
    type_info = f"[{metadata.param_type.value}]"
    example_text = f" Example: {metadata.example}" if metadata.example else ""

    return f"{metadata.name}{required_marker} {type_info} - {metadata.description}{example_text}"



"""Tests for CommandAutocomplete widget."""

import pytest
from awsui.autocomplete import CommandAutocomplete
from awsui.command_parser import CompletionContext


@pytest.fixture
def autocomplete():
    """Create autocomplete instance for testing."""
    commands = [
        "aws s3 ls",
        "aws s3 cp",
        "aws ec2 describe-instances",
        "aws lambda list-functions",
    ]
    categories = {
        "aws s3 ls": "s3/storage",
        "aws s3 cp": "s3/storage",
        "aws ec2 describe-instances": "ec2/compute",
        "aws lambda list-functions": "lambda/compute",
    }
    ac = CommandAutocomplete(commands, categories)
    return ac


def test_filter_commands_with_trailing_space(autocomplete):
    """Test filter_commands works with trailing space (strip bug fix)."""
    autocomplete.filter_commands("aws ", 4)
    assert autocomplete.display is True
    assert len(autocomplete.filtered_commands) > 0


def test_filter_commands_without_strip(autocomplete):
    """Test filter_commands doesn't strip query before checking startswith."""
    autocomplete.filter_commands("aws ", 4)
    # Should use intelligent filter, not fuzzy
    assert autocomplete.display is True


def test_smart_insert_full_command_replacement(autocomplete):
    """Test smart_insert replaces entire input for full commands."""
    current_value = "aws s3"
    cursor_pos = 6
    selection = "aws s3 ls"

    new_value, new_cursor = autocomplete.smart_insert_selection(
        current_value, cursor_pos, selection
    )

    assert new_value == "aws s3 ls"
    assert new_cursor == 9
    assert new_value.count("aws") == 1


def test_smart_insert_token_insertion(autocomplete):
    """Test smart_insert correctly inserts token at cursor position."""
    current_value = "aws s3 "
    cursor_pos = 7
    selection = "ls"

    new_value, new_cursor = autocomplete.smart_insert_selection(
        current_value, cursor_pos, selection
    )

    assert new_value == "aws s3 ls "
    assert new_cursor == 10


def test_smart_insert_replaces_partial_token(autocomplete):
    """Test smart_insert replaces partial token correctly."""
    current_value = "aws s3 l"
    cursor_pos = 8
    selection = "ls"

    new_value, new_cursor = autocomplete.smart_insert_selection(
        current_value, cursor_pos, selection
    )

    assert new_value == "aws s3 ls "
    assert new_value.endswith("ls ")


def test_smart_insert_cursor_in_middle_of_token(autocomplete):
    """Test smart_insert with cursor in middle of token."""
    current_value = "aws ec2 describe --instance-ids i-123"
    cursor_pos = 15  # In middle of "describe"
    selection = "describe-instances"

    new_value, new_cursor = autocomplete.smart_insert_selection(
        current_value, cursor_pos, selection
    )

    # Should replace entire "describe" token
    assert "describe-instances" in new_value
    assert "describe" not in new_value or "describe-instances" in new_value
    # Parameters should be preserved
    assert "--instance-ids i-123" in new_value


def test_smart_insert_preserves_text_after_token(autocomplete):
    """Test smart_insert preserves text after current token."""
    current_value = "aws s3 ls --region us-east-1"
    cursor_pos = 8  # After "ls"
    selection = "cp"

    new_value, new_cursor = autocomplete.smart_insert_selection(
        current_value, cursor_pos, selection
    )

    assert "--region us-east-1" in new_value
    assert "cp" in new_value


def test_fuzzy_match_exact_substring(autocomplete):
    """Test fuzzy matching with exact substring."""
    matched, score = autocomplete.fuzzy_match("aws s3 ls", "s3")
    assert matched is True
    assert score > 0


def test_fuzzy_match_scattered_letters(autocomplete):
    """Test fuzzy matching with scattered letters."""
    matched, score = autocomplete.fuzzy_match("describe-instances", "dscins")
    assert matched is True


def test_intelligent_filter_calls_parser(autocomplete):
    """Test intelligent filter uses parser for suggestions."""
    autocomplete.filter_commands("aws s3 ", 7)
    assert autocomplete.display is True
    # Should show s3 commands
    assert len(autocomplete.filtered_commands) > 0


def test_filter_commands_min_length(autocomplete):
    """Test filter_commands requires minimum length."""
    autocomplete.filter_commands("a", 1)
    assert autocomplete.display is False
    assert autocomplete.filtered_commands == []


def test_filter_commands_empty_input(autocomplete):
    """Test filter_commands with empty input."""
    autocomplete.filter_commands("", 0)
    assert autocomplete.display is False
    assert autocomplete.filtered_commands == []


def test_get_selected_command(autocomplete):
    """Test getting currently selected command."""
    autocomplete.filter_commands("aws s3 ", 7)
    autocomplete.highlighted = 0
    selected = autocomplete.get_selected_command()
    assert selected is not None
    assert selected in autocomplete.filtered_commands


def test_move_cursor_down(autocomplete):
    """Test moving selection down."""
    autocomplete.filter_commands("aws s3 ", 7)
    autocomplete.highlighted = 0
    initial = autocomplete.highlighted
    autocomplete.move_cursor_down()
    assert autocomplete.highlighted == initial + 1


def test_move_cursor_up(autocomplete):
    """Test moving selection up."""
    autocomplete.filter_commands("aws s3 ", 7)
    autocomplete.highlighted = 2
    autocomplete.move_cursor_up()
    assert autocomplete.highlighted == 1


def test_move_cursor_up_boundary(autocomplete):
    """Test moving selection up doesn't go below 0."""
    autocomplete.filter_commands("aws s3 ", 7)
    autocomplete.highlighted = 0
    autocomplete.move_cursor_up()
    assert autocomplete.highlighted == 0


def test_move_cursor_down_boundary(autocomplete):
    """Test moving selection down doesn't exceed list."""
    autocomplete.filter_commands("aws s3 ", 7)
    max_idx = len(autocomplete.filtered_commands) - 1
    autocomplete.highlighted = max_idx
    autocomplete.move_cursor_down()
    assert autocomplete.highlighted == max_idx



"""Tests for data models."""

from awsui.models import Profile


def test_profile_sso():
    """Test SSO profile creation."""
    profile = Profile(
        name="test-sso",
        kind="sso",
        account="111111111111",
        role="AdministratorAccess",
        region="ap-northeast-1",
        session="corp",
        source="/path/to/config"
    )

    assert profile["name"] == "test-sso"
    assert profile["kind"] == "sso"
    assert profile["account"] == "111111111111"
    assert profile["role"] == "AdministratorAccess"
    assert profile["region"] == "ap-northeast-1"
    assert profile["session"] == "corp"


def test_profile_assume_role():
    """Test assume role profile creation."""
    profile = Profile(
        name="test-assume",
        kind="assume",
        account="222222222222",
        role="MyRole",
        region="us-west-2",
        session=None,
        source="/path/to/config"
    )

    assert profile["name"] == "test-assume"
    assert profile["kind"] == "assume"
    assert profile["account"] == "222222222222"
    assert profile["role"] == "MyRole"
    assert profile["session"] is None


def test_profile_basic():
    """Test basic profile creation."""
    profile = Profile(
        name="default",
        kind="basic",
        account=None,
        role=None,
        region=None,
        session=None,
        source="/path/to/credentials"
    )

    assert profile["name"] == "default"
    assert profile["kind"] == "basic"
    assert profile["account"] is None
    assert profile["role"] is None